#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2015, IBM Corp.
# All rights reserved.
#
# Distributed under the terms of the BSD Simplified License.
#
# The full license is in the LICENSE file, distributed with this software.
# -----------------------------------------------------------------------------

"""
In-Database User defined functions
"""
from __future__ import print_function
from __future__ import unicode_literals
from __future__ import division
from __future__ import absolute_import

import inspect
import textwrap
from builtins import dict
from future import standard_library

from nzpyida.ae import shaper, result_builder

standard_library.install_aliases()


class NZFunGroupedApply(object):

    def __init__(self, df, index, output_signature, output_table=None, fun_ref=None,  code_str=None,  fun_name=None, columns=None, merge_output_with_df=False, id='ID'):
        """
        Constructor for tapply
        """
        self.table_name = df.internal_state.current_state
        self.df = df
        self.db = df._idadb
        self.fun = fun_ref
        self.fun_name = fun_name
        self.id = id

        self.code_str= code_str
        self.merge_output= merge_output_with_df

        self.index = index
        self.output_table = output_table
        self.output_signature = output_signature
        self.parallel = True

        # convert the columns index object to a list
        if columns:
            self.columns=columns
        else :
            self.columns = self.df.columns.tolist()
        print(self.columns)

    def get_result(self):
        if self.code_str and self.fun_name is None:
            raise Exception("fun_name is required")
        # we need a comma separated string of column values
        #columns_string = ",".join(self.columns)
        # print(columns_string)



        if self.code_str:
            code_string = self.build_udtf_shaper_ae_code_fun_as_str(self.code_str, self.fun_name, self.columns,
                                                                    self.output_signature)
        else:
            code_string = self.build_udtf_shaper_ae_code_fun_as_ref(self.fun, self.columns, self.output_signature)



        compressions_string = result_builder.compress_columns_string(self.columns)

        query = ""
        # send the code as dynamic variable to ae function
        columns_string = compressions_string + ",'CODE_TO_EXECUTE=" + "\"" + code_string + "\"" + "'"
        if self.parallel is False:
            ae_name = "nzpyida..py_udtf_host"
            query = "select ae_output.* from " + \
                    " (select * from " + self.db_name + ") as input_t" + \
                    ", table with final (" + ae_name + columns_string + ")) as ae_output"
        else:
            ae_name = "nzpyida..py_udtf_any"
            query = "(select row_number() over (partition by " + self.index + " order by " + self.index + ") as  rn,  count(*)  over (partition" + \
                    " by " + self.index + ") as   ct,    " + self.table_name + ".*   from " + self.table_name + ") as input_t"
            query = "select ae_output.* from " + \
                    query + \
                    ", table with final (" + ae_name + "(rn,ct," + columns_string + ")) as ae_output"
        result = result_builder.build_result(self.output_table, self.merge_output, self.db, self.df,
                                             self.output_signature, self.table_name, query, self.id)
        return result



    def build_udtf_shaper_ae_code_fun_as_str(self, code_str, fun_name, columns, output_signature):
        # we need extra single quotes for correct escaping

        fun_code = code_str
        fun_code = fun_code.replace("'", "''")


        if self.parallel is False:
            base_code = self.get_base_shaper_host(columns, fun_name, output_signature)
        else:
            base_code = shaper.get_base_shaper_groupedapply(columns, fun_name, output_signature)

        run_string = textwrap.dedent(""" BaseShaperUdtf.run()""")

        final_code = base_code + "\n" + textwrap.indent(fun_code, '     ')
        final_code = final_code+"\n"+run_string



        return inspect.cleandoc(final_code)




    def build_udtf_shaper_ae_code_fun_as_ref(self, fun, columns, output_signature):


        return self.build_udtf_shaper_ae_code_fun_as_str(inspect.getsource(fun).lstrip(), fun.__name__, columns,
                                                         output_signature)




    def get_base_shaper_host(self, columns, fun_name, output_signature):

        output_signature_str = shaper.get_base_shaper(output_signature)





        code_string = """               import nzae
                                        import pandas as pd
                                        class BaseShaperUdtf(nzae.Ae):
                                             def _runUdtf(self):
                                               rows_list=[]
                                               for row in self:
                                                 rows_list.append(row)
                                                 #rows_list.append(str(self.getDatasliceId())+str(self.getHardwareId()))  
                                               df = pd.DataFrame(rows_list, columns=""" + str(columns) + """ )
                                               value = self.""" + fun_name + """(df)
                                               self.output(value)
                                               #self.output(str(len(rows_list)), str(rows_list[12000]))

                                             def _runShaper(self):
                                               """ + textwrap.indent(output_signature_str, '                      ') + """




                                               """

        code_string = code_string.replace("'", "''")
        return inspect.cleandoc(code_string)





from nzpyida import IdaDataBase, IdaDataFrame

from nzpyida.ae.install import NZInstall

idadb = IdaDataBase('fyre', 'admin', 'password')

nzinstall = NZInstall(idadb, package_name='scikit-learn')
print(nzinstall.getResultCode())




# -*- coding: utf-8 -*-
"""
Created on Mon Dec 14 11:31:52 2015

@author: efouche
"""
from __future__ import division
from __future__ import unicode_literals
from __future__ import print_function
from __future__ import absolute_import
from builtins import dict
from future import standard_library
standard_library.install_aliases()

from collections import OrderedDict

from nzpyida.internals import idadf_state
from nzpyida.utils import timed

import numpy as np 
import pandas as pd

from nzpyida.feature_selection.private import _check_input

@idadf_state
@timed
def chisquared(idadf, target = None, features = None, ignore_indexer=True):
    """
    Compute the Chi-Squared statistics coefficients between a set of features 
    and a set of target in an IdaDataFrame. 
    
    Parameters
    ----------
    idadf : IdaDataFrame
    
    target : str or list of str, optional
        A column or list of columns against to be used as target. Per default, 
        consider all columns
    
    features : str or list of str, optional
        A column or list of columns to be used as features. Per default, 
        consider all columns. 
    
    ignore_indexer : bool, default: True
        Per default, ignore the column declared as indexer in idadf
        
    Returns
    -------
    Pandas.DataFrame or Pandas.Series if only one target
    
    Notes
    -----
    Input columns as target and features should be categorical, otherwise 
    this measure does not make much sense. 
    
    Chi-squared as defined in 
    A Comparative Study on Feature Selection and Classification Methods Using 
    Gene Expression Profiles and Proteomic Patterns. (GIW02F006)
    
    The scalability of this approach is not very good. Should not be used on 
    high dimensional data. 
    
    Examples
    --------
    >>> idadf = IdaDataFrame(idadb, "IRIS")
    >>> chisquared(idadf)
    """
    # Check input
    target, features = _check_input(idadf, target, features, ignore_indexer)
    count_dict = dict()
    length = len(idadf)
    
    values = OrderedDict()
         
    for t in target:   
        if t not in values:
            values[t] = OrderedDict() 
        features_notarget = [x for x in features if (x != t)]
        
        ### Compute
        for feature in features_notarget:
            if feature not in values:
                values[feature] = OrderedDict()
            if t not in values[feature]:
                if t not in count_dict:
                    count = idadf.count_groupby(t)
                    count_serie = count["count"]
                    count_serie.index = count[t]
                    count_dict[t] = count_serie
            
                C = dict(count_dict[t])
                
                if feature not in count_dict:
                    count = idadf.count_groupby(feature)
                    count_serie = count["count"]
                    count_serie.index = count[feature]
                    count_dict[feature] = count_serie
                    
                R = dict(count_dict[feature])
                
                if (feature, t) not in count_dict:
                    count_dict[(feature, t)] = idadf.count_groupby([feature , t])
                
                count = count_dict[(feature, t)]
                
                chi = 0            
                for target_class in C.keys():
                    count_target = count[count[t] == target_class][[feature, "count"]]
                    A_target = count_target['count']
                    A_target.index = count_target[feature]
                    
                    for feature_class in A_target.index:
                        a = A_target[feature_class]
                        e = R[feature_class] * C[target_class] / length
                        chi += ((a - e)**2)/e
                
                values[t][feature] = chi   # chisquared is symmetric 
                if feature in target:
                    values[feature][t] = chi
        
    result = pd.DataFrame(values).fillna(np.nan)
    result = result.dropna(axis=1, how="all")
        
    if len(result.columns) > 1:
        order = [x for x in result.columns if x in features] + [x for x in features if x not in result.columns]
        result = result.reindex(order)
    
    if len(result.columns) == 1:
        if len(result) == 1:
            result = result.iloc[0,0]
        else:
            result = result[result.columns[0]].copy()
            result.sort_values(ascending = False)
        

    
    
    return result
    
    



from nzpyida import IdaDataBase, IdaDataFrame


from nzpyida.ae import NZFunGroupedApply


#

#nzpy dsn
dsn ={
    "database":"customer_churn",
     "port" :5480,
     "host" : "169.63.46.17",
     "securityLevel":0,
     "logLevel":0


}

#odbc dsn
#dsn='bank'


idadb = IdaDataBase(dsn, 'admin', 'password', verbose=True )


print(idadb)

idadf = IdaDataFrame(idadb, 'customer_churn')


print(idadf.dtypes)
code_str_host_spus = """def log_reg_ml(self, df):
            from sklearn.model_selection import cross_val_score
            from sklearn.impute import SimpleImputer
            from sklearn.linear_model import LogisticRegression
            from sklearn.model_selection import train_test_split

            from sklearn.preprocessing import LabelEncoder
            import numpy as np
            
            
            
            

            # data preparation
            imputed_df = df.copy()
            ds_size = len(imputed_df)
            temp_dict = dict()


            columns = imputed_df.columns

            for column in columns:
                if column=='ID':
                    continue

                if (imputed_df[column].dtype == 'float64' or imputed_df[column].dtype == 'int64'):
                  if imputed_df[column].isnull().sum()==len(imputed_df):
                     imputed_df[column] = imputed_df[column].fillna(0)

                  else :

                     imp = SimpleImputer(missing_values=np.nan, strategy='mean')
                     transformed_column = imp.fit_transform(imputed_df[column].values.reshape(-1, 1))         
                     imputed_df[column] = transformed_column

                if (imputed_df[column].dtype == 'object'):
                    # impute missing values for categorical variables
                    imp = SimpleImputer(missing_values=None, strategy='constant', fill_value='missing')
                    imputed_df[column] = imp.fit_transform(imputed_df[column].values.reshape(-1, 1))
                    imputed_df[column] = imputed_df[column].astype('str')
                    le = LabelEncoder()

                    le.fit(imputed_df[column])
                    # print(le.classes_)
                    imputed_df[column] = le.transform(imputed_df[column])
                    temp_dict[column] = le



            # Create a decision tree
            lr = LogisticRegression()
            X = imputed_df.drop(['EXITED'], axis=1)
            y = imputed_df['EXITED']
            X_train, X_test, y_train, y_test = train_test_split(X,y, test_size = 0.25, random_state=42, stratify=y)


            lr.fit(X_train, y_train)

            accuracy = lr.score(X_test, y_test)    
            #print(accuracy)



            pred_df = X_test.copy()


            y_pred= (lr.predict_proba(X_test)[:,1]>=0.2)

            pred_df['EXITED'] = y_pred
            pred_df['DATASET_SIZE'] = ds_size
            pred_df['CLASSIFIER_ACCURACY']=round(accuracy,2)
            




            original_columns = pred_df.columns

            for column in original_columns:

             if column in temp_dict:   
               pred_df[column] = temp_dict[column].inverse_transform(pred_df[column])
               #print(pred_df)

            def print_output(x):
                row = [x['CUSTOMERID'],x['EXITED'], x['DATASET_SIZE']]
                self.output(row)


            pred_df.apply(print_output, axis=1)






"""

output_signature = {'CUSTOMERID':'float', 'EXITED_PRED': 'int',  'DATASET_SIZE': 'int'}
import time

start = time.time()

nz_groupapply = NZFunGroupedApply(df=idadf, code_str=code_str_host_spus, index='GEOGRAPHY', fun_name="log_reg_ml",
                                  output_signature=output_signature, merge_output_with_df=True, id='CUSTOMERID')
# nz_groupapply = NZFunGroupedApply(df=idadf,  code_str=code_str_host_spus, index='LOCATION', fun_name="decision_tree_ml")
result = nz_groupapply.get_result()
result = result.as_dataframe()
print("Host+ SPUs execution - slicing on user selection -ML function for partitions within slices\n")
print(result)
end = time.time()
print(end - start)



from nzpyida import IdaDataBase, IdaDataFrame

from nzpyida.ae import NZFunGroupedApply

dsn ='bank'

idadb = IdaDataBase(dsn, 'admin', 'password')
print(idadb)


idadf = IdaDataFrame(idadb, 'stocks')
print(idadf.head())


query = 'select * from stocks'

df = idadf.ida_query(query)
print(df.dtypes)


code_str_host_spus = """def stocks_rf_ml(self, df):

    import numpy as np

    #how many days in future you need to predict
    future_days = -1

    from sklearn.impute import SimpleImputer
    from sklearn.metrics import mean_squared_error
    from sklearn.ensemble import RandomForestRegressor
    from sklearn.model_selection import train_test_split


    from sklearn.preprocessing import LabelEncoder

    temp_dict = dict()
    
    
    
    df['DATE'] = pd.to_datetime(df['DATE'])
    df = df.sort_values(by='DATE')
    df['DATE']=df['DATE'].dt.date

    # data preparation

    imputed_df = df.copy()
    
    
    # add the future close price column and shift by the required days

    imputed_df['FUTURE_CLOSE_PRICE'] = imputed_df['ADJCLOSE'].shift(future_days)



    # add technical indicators
    for n in [14,30,50,200]:

     # create the moving average indicator 
     imputed_df['ma'+str(n)] = imputed_df['ADJCLOSE'].ewm(span=n,adjust=False).mean()



    ds_size = len(imputed_df)
    #imputed_df.dropna(inplace=True)


    columns = imputed_df.columns
    for column in columns:

        if column=='ID':
            continue

        #impute missing values 
        # mean for numerical and 'missing' for categorical
        if (imputed_df[column].dtype == 'float64' or imputed_df[column].dtype == 'int64'):


            if imputed_df[column].isnull().sum()==len(imputed_df):

                imputed_df[column] = imputed_df[column].fillna(0)


            else :

                imp = SimpleImputer(missing_values=np.nan, strategy='mean')

                transformed_column = imp.fit_transform(imputed_df[column].values.reshape(-1, 1))


                imputed_df[column] = transformed_column


        if (imputed_df[column].dtype == 'object'):
            # impute missing values for categorical variables


            imp = SimpleImputer(missing_values=None, strategy='constant', fill_value='missing')
            imputed_df[column] = imp.fit_transform(imputed_df[column].values.reshape(-1, 1))
            imputed_df[column] = imputed_df[column].astype('str')

            le = LabelEncoder()

            le.fit(imputed_df[column])
                    # print(le.classes_)
            imputed_df[column] = le.transform(imputed_df[column])
            temp_dict[column] = le


    # Create a random forest regressor
    rf = RandomForestRegressor(n_estimators=200)

    X = imputed_df.drop(['FUTURE_CLOSE_PRICE'], axis=1)
    y = imputed_df['FUTURE_CLOSE_PRICE']

    X_train, X_test, y_train, y_test = train_test_split(X,y, test_size = 0.25, random_state=42)


    rf.fit(X_train,y_train)




    pred_df = X_test.copy()
    pred_df['FUTURE_CLOSE_PRICE'] = y_test

    y_pred = rf.predict(X_test)
    pred_df['FUTURE_CLOSE_PRICE_PRED'] =y_pred



    accuracy  = rf.score(X_test, y_test)    

    rms = mean_squared_error(y_test, y_pred)


    pred_df['DATASET_SIZE'] = ds_size
    pred_df['accuracy']=round(accuracy,2)
    pred_df['mean_square_error']=round(rms)


    #for all the columns that had label encoders, do an inverse transform

    original_columns = pred_df.columns

    for column in original_columns:

     if column in temp_dict:   
      pred_df[column] = temp_dict[column].inverse_transform(pred_df[column])

    #pred_df['DATE'] = pd.to_datetime(pred_df['DATE'])



    #pred_df = pred_df.sort_values(by='DATE')
    #pred_df['DATE']=pred_df['DATE'].dt.date

    def print_output(x):
                row = [x['ID'], x['FUTURE_CLOSE_PRICE'], x['FUTURE_CLOSE_PRICE_PRED'], x['DATASET_SIZE'], x['accuracy']]
               
                
                self.output(row)


    pred_df.apply(print_output, axis=1)



"""

output_signature= {'ID': 'float', 'FUTURE_CLOSE_PRICE': 'double', 'FUTURE_CLOSE_PRICE_PRED': 'double',
                   'DATASET_SIZE': 'int', 'ACCURACY': 'float'}


import time

start = time.time()

nz_groupapply = NZFunGroupedApply(df=idadf, code_str=code_str_host_spus, index='TICKER', fun_name="stocks_rf_ml",
                                  output_signature=output_signature, merge_output_with_df=True)

result = nz_groupapply.get_result()
result = result.as_dataframe()
print("Host+ SPUs execution - slicing on user selection -ML function for partitions within slices\n")
print(result)
end = time.time()
print(end - start)







#!/usr/bin/env python
# -*- coding: utf-8 -*-
#-----------------------------------------------------------------------------
# Copyright (c) 2015, IBM Corp.
# All rights reserved.
#
# Distributed under the terms of the BSD Simplified License.
#
# The full license is in the LICENSE file, distributed with this software.
#-----------------------------------------------------------------------------

"""
Test module for IdaDataFrameObjects
"""
from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
from future import standard_library
standard_library.install_aliases()


class Test_aggregation(object):

    # Don't forget to test the recursive way too

    def test_idadf_aggregation_1_column_add_int(idadf, df):
        pass

    def test_idadf_aggregation_1_column_mul_int(idadf, df):
        pass

    def test_idadf_aggregation_1_column_div_int(idadf, df):
        pass

    def test_idadf_aggregation_1_column_sub_int(idadf, df):
        pass

    def test_idadf_aggregation_1_column_floordiv_int(idadf, df):
        pass

    def test_idadf_aggregation_1_column_mod_int(idadf, df):
        pass

    def test_idadf_aggregation_1_column_neg_int(idadf, df):
        pass

    def test_idadf_aggregation_1_column_pow_int(idadf, df):
        pass

    def test_idadf_aggregation_X_column_add_int(idadf, df):
        pass

    def test_idadf_aggregation_X_column_mul_int(idadf, df):
        pass

    def test_idadf_aggregation_X_column_div_int(idadf, df):
        pass

    def test_idadf_aggregation_X_column_sub_int(idadf, df):
        pass

    def test_idadf_aggregation_X_column_floordiv_int(idadf, df):
        pass

    def test_idadf_aggregation_X_column_mod_int(idadf, df):
        pass

    def test_idadf_aggregation_X_column_neg_int(idadf, df):
        pass

    def test_idadf_aggregation_X_column_pow_int(idadf, df):
        pass


    def test_idadf_aggregation_1_column_add_ida(idadf, df):
        pass

    def test_idadf_aggregation_1_column_mul_ida(idadf, df):
        pass

    def test_idadf_aggregation_1_column_div_ida(idadf, df):
        pass

    def test_idadf_aggregation_1_column_sub_ida(idadf, df):
        pass

    def test_idadf_aggregation_1_column_floordiv_ida(idadf, df):
        pass

    def test_idadf_aggregation_1_column_mod_ida(idadf, df):
        pass

    def test_idadf_aggregation_1_column_neg_ida(idadf, df):
        pass

    def test_idadf_aggregation_1_column_pow_ida(idadf, df):
        pass

    def test_idadf_aggregation_X_column_add_ida(idadf, df):
        pass

    def test_idadf_aggregation_X_column_mul_ida(idadf, df):
        pass

    def test_idadf_aggregation_X_column_div_ida(idadf, df):
        pass

    def test_idadf_aggregation_X_column_sub_ida(idadf, df):
        pass

    def test_idadf_aggregation_X_column_floordiv_ida(idadf, df):
        pass

    def test_idadf_aggregation_X_column_mod_ida(idadf, df):
        pass

    def test_idadf_aggregation_X_column_neg_ida(idadf, df):
        pass

    def test_idadf_aggregation_X_column_pow_ida(idadf, df):
        pass

    def test_idadf_aggregation_error(idadf, df):
        pass





#!/usr/bin/env python
# -*- coding: utf-8 -*-
#-----------------------------------------------------------------------------
# Copyright (c) 2015, IBM Corp.
# All rights reserved.
#
# Distributed under the terms of the BSD Simplified License.
#
# The full license is in the LICENSE file, distributed with this software.
#-----------------------------------------------------------------------------

'''
This module provides a data set containing standardized fertility measure and
socio-economic indicators for each of 47 French-speaking provinces of
Switzerland at about 1888. As user, you can import the object 'swiss' which
is a pandas dataframe with 47 rows and the following fields:
    swiss['Fertility']
    swiss['Agriculture']
    swiss['Examination']
    swiss['Education']
    swiss['Catholic']
    swiss['Infant.Mortality']

Examples
--------
>>> from nzpyida.sampledata.swiss import swiss
'''

from __future__ import absolute_import
from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from future import standard_library
standard_library.install_aliases()
from os.path import dirname, join
import pandas as pd

swiss = pd.read_csv(join(dirname(__file__), 'swiss.txt'), index_col = 0)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#-----------------------------------------------------------------------------
# Copyright (c) 2015, IBM Corp.
# All rights reserved.
#
# Distributed under the terms of the BSD Simplified License.
#
# The full license is in the LICENSE file, distributed with this software.
#-----------------------------------------------------------------------------

"""
Test module for IdaDataFrameObjects
"""
from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
from future import standard_library
standard_library.install_aliases()

import pandas
import pytest
import six
from nzpyida.learn import KMeans
from flaky import flaky

class Test_DataBaseExploration(object):

    def test_idadb_current_schema(self, idadb):
        assert isinstance(idadb.current_schema, six.string_types)

    def test_idadb_show_tables(self, idadb):
        test = idadb.show_tables()
        assert isinstance(test, pandas.core.frame.DataFrame)
        assert (list(test.columns) == ['TABSCHEMA','TABNAME','OWNER','TYPE'])

    def test_idadb_show_tables_all(self, idadb):
        test = idadb.show_tables(True)
        assert isinstance(test, pandas.core.frame.DataFrame)
        assert (list(test.columns) == ['TABSCHEMA','TABNAME','OWNER','TYPE'])

    def test_idadb_show_models(self, idadb):
        df = idadb.show_models()
        assert isinstance(df, pandas.core.frame.DataFrame)
        if not df.empty:
            show_model_columns = ['MODELNAME', 'OWNER', 'CREATED', 'STATE',
                                  'MININGFUNCTION', 'ALGORITHM', 'USERCATEGORY']
            if not idadb._is_netezza_system():
                show_model_columns =  ['MODELSCHEMA'] + show_model_columns
            assert (list(df.columns) == show_model_columns)

    def test_idadb_exists_table_or_view_positive0(self, idadb, idadf, idaview):
        assert(idadb.exists_table_or_view(idadf.name) == 1)
        assert(idadb.exists_table_or_view(idaview.name) == 1)

    @pytest.mark.skipif("'netezza' in config.getvalue('jdbc') or config.getvalue('hostname') != ''")
    def test_idadb_exists_table_or_view_positive(self, idadb, idadf, idaview, idageodf_county_view):
        assert(idadb.exists_table_or_view(idadf.name) == 1)
        assert(idadb.exists_table_or_view(idaview.name) == 1)
        assert(idadb.exists_table_or_view(idageodf_county_view.name) == 1)

    def test_idadb_exists_table_or_view_negative(self, idadb):
        assert(idadb.exists_table_or_view("NOT_EXISTING_DATA_FRAME_130530496_4860385960") == 0)
        assert(idadb.exists_table_or_view("NOT_EXISTING_130530496_4860385960") == 0)

    #def test_idadb_exists_table_or_view_error(self, idadb):
    #    with pytest.raises(TypeError):
    #        idadb.exists_table_or_view("ST_INFORMTN_SCHEMA.ST_UNITS_OF_MEASURE")
    # Note: We cannot assume that "ST_INFORMTN_SCHEMA.ST_UNITS_OF_MEASURE" exists and is of type "A"

    def test_idadb_exists_table_positive(self, idadb, idadf):
        assert(idadb.exists_table(idadf.name) == 1)

    def test_idadb_exists_table_negative(self, idadb):
        assert(idadb.exists_table("NOT_EXISTING_FRAME_130530496_4860385960") == 0)

    def test_idadb_exists_table_error(self, idadb, idaview):
        with pytest.raises(TypeError):
            idadb.exists_table(idaview.name)

    def test_idadb_exists_view_positive(self, idadb, idaview):
        assert(idadb.exists_view(idaview.name) == 1)

    def test_idadb_exists_view_negative(self, idadb):
        assert(idadb.exists_view("NOT_EXISTING_VIEW_130530496_4860385960") == 0)

    def test_idadb_exists_view_error(self, idadb, idadf):
        with pytest.raises(TypeError):
            idadb.exists_view(idadf.name)

    def test_idadb_exists_model_positive(self, idadb, idadf_tmp):
        idadb.add_column_id(idadf_tmp, destructive=True)
        # Create a simple KMEANS model
        kmeans = KMeans(n_clusters=3, modelname="MODEL_58979457385")
        kmeans.fit(idadf_tmp)
        assert(idadb.exists_model("MODEL_58979457385") == 1)
        try :
            idadb.drop_model(kmeans.modelname)
        except : pass

    @pytest.mark.skipif("'netezza' in config.getvalue('jdbc') or config.getvalue('hostname') != ''")
    def test_idadb_exists_model_with_schema_positive(self, idadb, idadf_tmp):
        idadb.add_column_id(idadf_tmp, destructive=True)
        # Create a simple KMEANS model
        kmeans = KMeans(n_clusters=3, modelname="MYSCHEMA.MODEL_45738558979")
        kmeans.fit(idadf_tmp)
        assert(idadb.exists_model("MYSCHEMA.MODEL_45738558979") == 1)
        try :
            idadb.drop_model(kmeans.modelname)
        except : pass

    @pytest.mark.skipif("'netezza' in config.getvalue('jdbc') or config.getvalue('hostname') != ''")
    def test_idadb_exists_model_with_schema_positive_mixed_case(self, idadb, idadf_tmp):
        idadb.add_column_id(idadf_tmp, destructive=True)
        # Create a simple KMEANS model
        kmeans = KMeans(n_clusters=3, modelname="mySchema.Model_85584573979")
        kmeans.fit(idadf_tmp)
        assert(idadb.exists_model("mySchema.Model_85584573979") == 1)
        assert(idadb.exists_model("MYSCHEMA.MODEL_85584573979") == 1)
        assert(idadb.exists_model("myschema.model_85584573979") == 1)
        try :
            idadb.drop_model(kmeans.modelname)
        except : pass

    @flaky
    def test_idadb_exists_model_negative(self, idadb):
        if idadb.exists_model("MODEL_58979457385"):
            idadb.drop_model("MODEL_58979457385")
        assert(idadb.exists_model("MODEL_58979457385") == 0)

    def test_idadb_exists_model_with_schema_negative(self, idadb):
        if idadb.exists_model("SCHEMA_58979457385.MODEL_58979457385"):
            idadb.drop_model("SCHEMA_58979457385.MODEL_58979457385")
        assert(idadb.exists_model("SCHEMA_58979457385.MODEL_58979457385") == 0)

    def test_idadb_exists_model_error(self, idadb, idadf):
        with pytest.raises(TypeError):
            idadb.exists_model(idadf.name)

    def test_idadb_is_table_or_view_positive(self, idadb, idadf, idaview):
        assert(idadb.is_table_or_view(idadf.name) == 1)
        assert(idadb.is_table_or_view(idaview.name) == 1)

    #def test_idadb_is_table_or_view_negative(self, idadb):
    #    assert(idadb.is_table_or_view("ST_INFORMTN_SCHEMA.ST_UNITS_OF_MEASURE") == 0)

    def test_idadb_is_table_or_view_error(self, idadb):
        with pytest.raises(ValueError):
            idadb.is_table_or_view("NOT_EXISTING_DATA_FRAME_130530496_4860385960")

    def test_idadb_is_table_positive(self, idadb, idadf):
        assert(idadb.is_table(idadf.name) == 1)

    def test_idadb_is_table_negative(self, idadb, idaview):
        assert(idadb.is_table(idaview.name) == 0)

    def test_idadb_is_table_error(self, idadb):
        with pytest.raises(ValueError):
            idadb.is_table("NOT_EXISTING_DATA_FRAME_130530496_4860385960")

    def test_idadb_is_view_positive(self, idadb, idaview):
        assert(idadb.is_view(idaview.name) == 1)

    def test_idadb_is_view_negative(self, idadb, idadf):
        assert(idadb.is_view(idadf.name) == 0)

    def test_idadb_is_view_error(self, idadb):
        with pytest.raises(ValueError):
            idadb.is_view("NOT_EXISTING_VIEW_130530496_4860385960")

    def test_idadb_is_model_positive(self, idadb, idadf_tmp):
        idadb.add_column_id(idadf_tmp, destructive = True)
        # Create a simple KMEANS model
        kmeans = KMeans(n_clusters = 3)
        kmeans.fit(idadf_tmp)
        assert(idadb.is_model(kmeans.modelname) == 1)
        try : idadb.drop_model(kmeans.modelname)
        except : pass

    def test_idadb_is_model_negative(self, idadb, idadf, idaview):
        assert(idadb.is_model(idadf.name) == 0)
        assert(idadb.is_model(idaview.name) == 0)
        if not idadb._is_netezza_system():
            assert(idadb.is_model("ST_INFORMTN_SCHEMA.ST_UNITS_OF_MEASURE") == 0)

    def test_idadb_is_model_error(self, idadb):
        with pytest.raises(ValueError):
            idadb.is_model("NOTEXISTINGOBJECT_496070383095079384063739509")

# List of functions that do not need to be tested :
# i.e. the execution of everything here rely on it
# _prepare_and_execute
# _call_stored_procedure
# _autocommit
# _check_connection
# _exists
# _is
# _drop





#!/usr/bin/env python
# -*- coding: utf-8 -*-
#-----------------------------------------------------------------------------
# Copyright (c) 2015, IBM Corp.
# All rights reserved.
#
# Distributed under the terms of the BSD Simplified License.
#
# The full license is in the LICENSE file, distributed with this software.
#-----------------------------------------------------------------------------

"""
The module contains the function that is used to modify or
create columns in an IdaDataFrame based on aggreation
"""

from __future__ import print_function
from __future__ import unicode_literals
from __future__ import division
from __future__ import absolute_import
from builtins import str
from future import standard_library
standard_library.install_aliases()

from numbers import Number
from collections import OrderedDict

import nzpyida
from nzpyida.exceptions import IdaDataBaseError

def aggregate_idadf(idadf, method, other, swap = False):
    """
    Modify internal state variables to represent the aggregation of columns
    of an IdaDataFrame or IdaSeries in ibmdbpy.

    The following comparison operators are supported: +, \*, /, -, //, %, \*\*.

    The syntax is similar to Pandas.

    Parameters
    ----------
    idadf : IdaDataFrame or IdaSeries
        IdaDataFrame or IdaSerie on the left (if swap is False)
    method : str
        Aggregation method that is computed: the following values are 
        admissible: "add","mul","div","sub","floordiv","mod","neg","pow"
    other: Number or IdaDataFrame or IdaSeries
        Another object that idadf will be aggregated with (on the right if swap is False).
    swap : bool, default: False
        Internally used to handle cases where the call is made reflexively,
        that is when the main IdaDataFrame/IdaSeries is not on the left.
        If swap is True, this also implies that other is not of type IdaDataFrame/IdaSeries.

    Returns
    -------
    Aggregated IdaDataFrame or IdaSeries

    Raises
    ------
    ValueError
        Aggregation method not supported.
    TypeError
         Type not supported for aggregation.

    Examples
    --------
    >>> idairis['SepalLength'] = idairis['SepalLength'] * 2
    ...

    Notes
    -----
    It is not possible to create aggregations between columns that are stored 
    in different Db2 Warehouse tables.

    """
    def swap_manager(left, right, swap = False):
        if swap:
            left, right = right, left
        return (left, right)

    #Swap values in case of reflexive call
    # TODO : Override in IdaSeries instead of including the logic here.
    if swap:
        idadf, other = other, idadf


    if self._idadb._is_netezza_system():
        power_function = " POW"
    else:
        power_function = " POWER"

    simplemethod = {"add": " + ", "mul": " * ",  "div": " / ", "sub": " - "}
    complexmethod = {"floordiv" : " FLOOR(%s/%s) ",
                     "mod" : " MOD(%s,%s) ",
                     "neg" : " -%s%s ",
                     "pow" : power_function + "(%s,%s)"} # overflow risk, to handle

    all_methods = list(simplemethod.keys())+list(complexmethod.keys())
    if method not in all_methods:
        raise ValueError("Admissible values for method argument are %s." %str(all_methods)[1:-1])

    columndict = OrderedDict()

    if isinstance(idadf, nzpyida.IdaDataFrame):

        for index, column in enumerate(idadf.internal_state.columndict.keys()):
            column_value = idadf.internal_state.columndict[column]
            if other is None: # this is for now just the neg case
                left, right = swap_manager(column_value, '')
            elif isinstance(other, Number):
                left, right = swap_manager(column_value, other, swap)
            elif isinstance(other, nzpyida.IdaSeries):
                left, right = swap_manager(column_value, "%s"%list(other.internal_state.columndict.values())[0], swap)
            elif isinstance(other, nzpyida.IdaDataFrame):
                if len(idadf.columns) != len(other.columns):
                    if len(other.columns) != 1:
                        raise IdaDataBaseError("Number of columns of other "+
                                               "IdaDataFrame should be either "+
                                               "equal to aggregated IdaDataFrame"+
                                               "or equal to 1.")
                    left, right = swap_manager("%s"%column_value, "%s"%list(other.internal_state.columndict.values())[0], swap)
                else:
                    left, right = swap_manager("%s"%column_value, "%s"%list(other.internal_state.columndict.values())[index], swap)
            else:
                raise TypeError("Aggregation method not supported. Unsupported type for aggregation: %s"%type(other))

            if method in simplemethod:
                columndict[column] = "(%s%s%s)"%(left, simplemethod[method], right)
            elif method in complexmethod:
                agg = complexmethod[method] %(left, right)
                columndict[column] = "(%s)"%agg

        newidadf = idadf._clone()
        for key,value in columndict.items():
            newidadf.internal_state.columndict[key] = value

        newidadf.internal_state.update()
        # REMARK: Don't need to reset some attributes ?
        return newidadf

    if isinstance(idadf, nzpyida.IdaSeries):
        columnname = idadf.internal_state.columndict.keys()[0]
        if other is None: # this is for now just the neg case
            left, right = swap_manager("\"%s\""%columnname, '')
        elif isinstance(other, Number):
            left, right = swap_manager("\"%s\""%columnname, other, swap)
        elif isinstance(other, nzpyida.IdaSeries):
            left, right = swap_manager("\"%s\""%columnname, "\"%s\""%other.columns[0], swap)
        else:
            raise TypeError("Type not supported for aggregation: " + str(type(other)))

        if method in simplemethod:
            columndict[columnname] = "(%s%s%s)"%(left, simplemethod[method], right)
        elif method in complexmethod:
            agg = complexmethod[method] %(left, right)
            columndict[columnname] = "(%s)"%agg

        newidaseries = idadf._clone()
        newidaseries.internal_state.columndict[key] = columndict[columnname]
        newidaseries.internal_state.update()
        return newidaseries


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#-----------------------------------------------------------------------------
# Copyright (c) 2015, IBM Corp.
# All rights reserved.
#
# Distributed under the terms of the BSD Simplified License.
#
# The full license is in the LICENSE file, distributed with this software.
#-----------------------------------------------------------------------------

"""
Test module for connexion related functions of IdaDataFrame
"""
from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
from future import standard_library
standard_library.install_aliases()

import pandas

class Test_Import(object):

    def test_idadf_as_dataframe(self, idadf):
        tmp = idadf.as_dataframe() # For avoiding overhead off loading several time
        assert isinstance(tmp, pandas.core.frame.DataFrame)
        assert list(tmp.columns) == list(idadf.columns)
        assert list(tmp.index) == list(idadf.index)
        assert tmp.name == idadf.tablename

class Test_ConnexionManagement(object):
    def test_idadf_save_as(self, idadf):
        pass

    def test_idadf_commit(self, idadf):
        pass

    def test_idadf_rollback(self, idadf):
        pass

    def test_idadf_close(self, idadf):
        pass

    def test_idadf_reopen(self, idadf):
        pass


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#-----------------------------------------------------------------------------
# Copyright (c) 2015, IBM Corp.
# All rights reserved.
#
# Distributed under the terms of the BSD Simplified License.
#
# The full license is in the LICENSE file, distributed with this software.
#-----------------------------------------------------------------------------

"""
Definition of internal tools for non-destructive modification of IdaDataFrames.
"""

# Python 2 compatibility
from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
from builtins import str
from builtins import range
from future import standard_library
standard_library.install_aliases()

from functools import wraps, partial
from numbers import Number
from collections import OrderedDict

from lazy import lazy
import six

import nzpyida

from copy import deepcopy

def idadf_state(function=None, force=False):
    """
    View management for IdaDataFrame.
    This decorator creates the view reflecting the current state of an
    IdaDataFrame, so that the operation it decorates can work on this "virtual"
    version.
    """
    if function is None:
        return partial(idadf_state, force=force)
    @wraps(function)
    def wrapper(self, *args, **kwds):
        # Note: 5 is arbitrary. Maybe not the most relevant threshold
        # Means that not view should be created if the idadataframe was 
        # not modified at least 5 times. 
        if (len(self.internal_state.views) < 5)&(force is False):
            # Here, avoid creating a view but putting in the stack the string
            # that corresponds to the state of the idadataframe
            if self._idadb._is_netezza_system():
                asTemp = " AS TEMP"
            else:
                asTemp = ""
            if self.internal_state.views:
                self.internal_state.viewstack.append("(" + self.internal_state.get_state() + ")" + asTemp)
            try:
                result = function(self, *args, **kwds)
            except:
                raise
            finally:
                if self.internal_state.views:
                    if self.internal_state.viewstack:
                        self.internal_state.viewstack.pop()
        else:
            self.internal_state._create_view()
            try:
                result = function(self, *args, **kwds)
            except:
                raise
            finally:
                self.internal_state._delete_view()
        return result
    return wrapper

class InternalState(object):
    """
    Records and maintains an internal state of an IdaDataFrame.
    This class is used internally to manage modified version of IdaDataFrames
    without modifying the real version In-DataBase.
    """
    def __init__(self, idadf):
        """
        Constructor for InternalSate objects. InternalState objects shall be
        created only by IdaDataFrame/IdaSeries objects.

        Parameters
        ----------
        idadf : str
            IdaDataFrame to which self relates.

        Attributes
        ----------
        _idadf : IdaDataFrame
            IdaDataFrame to which self relates.
        name : str
            In database Name of the idaobject its belongs to.
        ascending : bool, default: True
            Indicate if the dataframe should be sorted in ascending or descending order.
        columns: str, default: '*'
            Columns that are selected in the IdaDataFrame, the order determine
            the order they are displayed too. Follow this model : ""Col1", "Col2", "Col3"".
        order: str, default: ''
            "Order by" part of a corresponding SQL query. Follow this model :
            " ORDER BY "Col1", "Col2", "Col3" ASC".
        _views : list of str
            List of nested SQL queries that describe the state of the
            IdaDataFrame. String that are stored  in this list may not be
            modified.
        _cumulative : str
            Cumulative represent the most current nested SQL queries. The main
            different with those in _views is that cumulative may still be
            modified. When the state of the IdaDataFrame is modified in the way
            that it is not possible to modify cumulative without violating the
            order the operation where may, the cumulative function is stored
            in _views, and a new cumulative gets created.
        viewstack : list of str
            Stack of view names that are created dynamically. Used to memorize
            the name of the temporary views so that they can get  deleted when
            the operation is finished.
        """
        self._idadf = idadf
        self.name = idadf._name
        self.tablename = idadf.tablename
        self.ascending = True # per default
        self.index = None
        self.order = None

        self._views = []
        self._cumulative = []
        self.viewstack = []

    @lazy
    def columndict(self):
        """
        Returns an ordered dictionnary that contains available columns as keys
        and their "real" expression as values, exactly in the way they should
        appear in modified SQL queries for creating views.
        """
        #return OrderedDict((cols,"\""+cols+"\"") for cols in self._idadf.columns)
        columns = deepcopy(OrderedDict((cols,"\""+cols+"\"") for cols in self._idadf.columns))
        
        #Remove the quotation marks enclosing DB2GSE functions
        #ibmdbpy stores and keeps track of columns internally enclosing them
        #with double quotation marks, but they must be removed for functions, 
        #so that the SQL query is interpreted correctly
        
        #TODO: Surround the geometries with ST_AsText()
        #so that the database retrieves geometries as CLOB with WKT instead 
        #of a geometry (which internally is a STRUCT type).
        #Although this STRUCT type has the method getSubString() of CLOB types, 
        #it makes jaydebeapi raise a warning of no mapping of STRUCT type
        #Pending to do this because knowing the type of a column by its name 
        #requires accessing the dtypes attribute, which is usually erased after
        #manipulating the Ida(Geo)DataFrame. In order to do this, refreshing
        #instead of erasing dtypes attribute would be a better approach

        for column in columns.keys():
            #if the column is an ST_ function
            if columns[column].find('ST_') == 0:
                columns[column] = columns[column][1:-1]
                
        return columns

    @property
    def views(self):
        """
        Returns the set of all views, including the cumulating view.
        """
        return self._views + self._cumulative

    @property
    def current_state(self):
        """
        If some temporary view have been created, i.e. the InternalState was
        modified and viewstack in not empty, returns the name of the last
        created view, otherwise returns the name of the table.
        """
        if self.viewstack:
            return self.viewstack[-1]
        else:
            return self.name

    def _create_view(self, viewname = None):
        """
        Creates a view that represent the current state of the idea dataframe

        Arguments
        ---------
        viewname : str, optional
            Name of the view to be created, if not given, then a unique name
            view will be generated.
        """
        if not self.views : return
        if viewname:
            view = viewname
        else:
            prefix = "TEMP_VIEW_%s_"%(self.tablename)
            view = self._idadf._idadb._get_valid_viewname(prefix)

        query = "CREATE VIEW \"%s\" AS (%s)"%(view, self.get_state())
        self._idadf._prepare_and_execute(query, autocommit = True)
        self.viewstack.append(view)
        return view

    def _delete_view(self, viewname = None):
        """
        Deletes a view related to the internal state

        Arguments
        ---------
        viewname : str, optional
            Deletes the view that has this name. If no viewname is given,
            deletes the most recently created view, i.e. the one on the stack.
        """
        if viewname:
            view = viewname
        else:
            if self.viewstack:
                view = self.viewstack.pop()
            else:
                return
        query = "DROP VIEW \"" + view + "\""
        self._idadf._prepare_and_execute(query, autocommit = True)
        return

    def __del__(self):
        """
        Overriding the destructor for InternalState object.
        Making sure all the views in the stack are deleted before deleting self.
        """
        if self.viewstack:
            while self.viewstack:
                self._delete_view(self.viewstack.pop())

    def update(self, filter_query=None):
        """
        Update _views and/or _cumulative with an SQL select expression that
        corresponds to the lastest modification of the internal state.
        Determines automatically if it is enough to replace _cumulative
        or to store it in _views and create a new _cumulative view.
        The situation, in which we should stop cumulating are :
        Filter, order, selection (of rows)
        """
        if isinstance(filter_query, nzpyida.filtering.FilterQuery):
            query = filter_query.query
            #self.stop_cumulating_columns() # not sure ?
            self._views = self._views + self._cumulative
            self._cumulative = []
            self._views.append(query)
        elif self.order is not None:
            # Assumption : order is the only modification
            query = "SELECT * FROM (SELECT * FROM %s"+self.get_order()+") AS TEMP"
            # self.order = None
            #self.stop_cumulating_columns()
            self._views = self._views + self._cumulative
            self._cumulative = []
            self._views.append(query)
        else:
            if self.index is None:
                query = ("SELECT " + self.get_columns() + " FROM %s")
                self._cumulative = [query]
            else:
                if isinstance(self.index, Number):
                    indexstring = " = " + str(self.index)
                elif isinstance(self.index, six.string_types):
                    if "'" in self.index:
                        self.index = self.index.replace("'", "\'")
                    indexstring = " = '" + str(self.index) + "'"
                elif isinstance(self.index, list):
                    indexstring = " in (" + str(self.index)[1:-1] + ")" #[1:-1] to strip the brackets
                elif isinstance(self.index, slice):
                    start = self.index.start
                    if not start:
                        start = 0
                    stop = self.index.stop
                    if not self.index.stop:
                        stop = self._idadf.shape[0]
                    if self.index.step:
                        # TODO: Improve this by using a modulo operation instead of listing
                        # This would prevent SQL overflow for big datasets
                        indexlist = list(range(start, stop)[start:stop:self.index.step])
                        indexstring = " in (" + str(indexlist)[1:-1] + ")"
                    else:
                        indexstring = " BETWEEN " + str(start) + " AND " + str(stop)

                columns = self.get_columns()
                #columns = self._idadf._get_columns()
                
                #columns = "\"" + "\",\"".join(set(self._idadf._get_columns())|set(self._idadf.columns)) + "\"" # TODO: Check if it is good
                #if columns == "*":
                    #columns = "\"" + "\",\"".join(self._idadf.columns) + "\""

                if self._idadf.indexer:
                    query = ("SELECT " + columns + " FROM %s AS TEMP1 WHERE " +
                             self._idadf.indexer + indexstring)
                else:
                    if self._idadf._idadb._is_netezza_system():
                        order_by = "ORDER BY NULL"
                    else:
                        order_by = ""
                    query = ("SELECT " + columns + " FROM (SELECT TEMP1.*, "+
                            "(ROW_NUMBER() OVER(" + order_by + ")-1) AS RN FROM %s AS TEMP1) AS TEMP2 "+
                            "WHERE RN " + indexstring)

                self.index = None

                #self.stop_cumulating_columns()
                self._views = self._views + self._cumulative
                self._cumulative = []
                self._views.append(query)


    def get_state(self):
        """
        Returns the sql query corresponding to the current state of the
        IdaDataFrame. The query is created by nesting of all strings in views
        (which is the concatenation of _views and _cumulative).
        """
        if not self.views:
            return ("SELECT " + self.get_columns() + " FROM " + self.name)

        query = "%s"
        for index, view in enumerate(self.views[::-1]):
            if index == (len(self.views)-1):
                view = view % self.name
            if index != 0:
                view = "(" + view + ")"
            query = query %(view)
        return query

    def get_columns(self):
        """
        Returns a string containing selected columns in the current state.
        For each column in in columndict, column expression are either :
            * The value of the column if it was unchanged ("<CONAME>")
            * The value of the column following by " AS \"<COLNAME>\""
        """
        columns = []
        for key,value in self.columndict.items():
            if value[1:-1] == key:
                columns.append(value)
            else:
                columns.append("%s AS \"%s\""%(value, key))

        return ",".join(columns)


    def get_order(self):
        """
        Returns the "Order by" clause of a corresponding SQL query, according
        to the value of the attribute order. If order is None, returns an
        empty string.

        Examples
        --------
        >>> self.get_order()
        " ORDER BY "Col1" ASC, "Col2" ASC, "Col3" ASC"
        """
        if not self.order:
            return ''
        else:
            if self.ascending:
                return (" ORDER BY \"" + "\" ASC, \"".join(self.order) + "\" ASC")
            else:
                return (" ORDER BY \"" + "\" DESC, \"".join(self.order) + "\" DESC")

    def is_default(self):
        """
        Checks if the InternalState object corresponds to the default values.

        Returns
        -------
        bool
            1 : The InternalState has default values
            0 : The InternalState is modifed
        """
        return self.get_state() == InternalState(self.name).get_state()

    def set_order(self, order, ascending):
        """
        Changes order in the internal state of the IdaDataframe.
        """
        print("set_order " + order[0])
        self.order = order
        self.ascending = ascending

    def reset_order(self):
        """
        Resets the state to original order. Delete all views that have an ORDER
        clause. Based on the assumption that ORDER is the only modification
        of those queries.
        """
        # This is based on the assumption than no other modifications than
        # sorting was done when sorting the idadf
        self._views = [view for view in self._views if "ORDER BY" not in view]

    def stop_cumulating_columns(self):
        # TODO: do not use anymore ??
        # You actually don't want to use column that you have not selected anymore ? 
        # except if you want to sum maybe ...
        """
        Management of views to ensure that some specific operations such as
        Filter, order, selection are registred in the list of views so that
        the real order of operation is respected.
        Here is what the function does:
        If _cumulative is not None,
        then the _cumulative is augmented with all available columns in the
        database, if any and then appended in _views.
        _cumulative is then reset to new query that only select the columns
        that were selected on that point. This is made so, because it allows
        users to still use all available columns in the table, even if they are
        not selected in the current state.
        """
        # TODO: Improve documentation for this method (experimental)

        # Add all columns available originally in the table in the history
        if self._cumulative:
            all_columns = self._idadf._get_all_columns_in_table()
            deleted_columns = []
            for column in all_columns:
                if column not in list(self.columndict.keys()):
                    deleted_columns.append(column)

            if deleted_columns:
                for column in deleted_columns:
                     self.columndict[column] = ("\""+column+"\"")
                self._cumulative = ["SELECT " + self.get_columns() + " FROM %s"]

            self._views = self._views + self._cumulative
            self._cumulative = []

            # Select only the columns that were available in the representation
            # Those 2 steps are made so to enable users to still use columns
            # that were not selected anymore in the current table representation
            # but still existing in the idadataframe

            if deleted_columns:
                for column in deleted_columns:
                    del self.columndict[column]
                self.columndict = OrderedDict((cols,"\""+cols+"\"") for cols in self.columndict)
                self._cumulative = ["SELECT " + self.get_columns() + " FROM %s"]
            else:
                self.columndict = OrderedDict((cols,"\""+cols+"\"") for cols in self.columndict)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#-----------------------------------------------------------------------------
# Copyright (c) 2015, IBM Corp.
# All rights reserved.
#
# Distributed under the terms of the BSD Simplified License.
#
# The full license is in the LICENSE file, distributed with this software.
#-----------------------------------------------------------------------------

"""
Test module for IdaDataFrameObjects
"""
from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
from future import standard_library
standard_library.install_aliases()

from numbers import Number

import pandas
import pytest
import six

from nzpyida import IdaDataBase, IdaDataFrame
from nzpyida.exceptions import IdaDataBaseError

class Test_ConnectToDB(object):

    def test_idadb_attr(self, idadb):

        if idadb._con_type == 'nzpy':
            assert isinstance(idadb.data_source_name, dict)
        else:
            assert isinstance(idadb.data_source_name, six.string_types)
        assert isinstance(idadb._con_type, six.string_types)
        assert isinstance(idadb._idadfs, list)
        if idadb._con_type =='odbc':
            assert(isinstance(idadb._connection_string, six.string_types))
        elif idadb._con_type == 'jdbc':
            assert(isinstance(idadb._connection_string, six.string_types) or isinstance(idadb._connection_string, list))
        elif idadb._con_type == 'nzpy':
            assert(isinstance(idadb._connection_string, dict))
        assert(type(idadb._con).__name__ in ['Connection','instance'])
        assert(idadb._con_type in ["odbc", "jdbc", "nzpy"])

    def test_idadb_instance_fail(self, idadb):

        if idadb._con_type == 'odbc':
            with pytest.raises(IdaDataBaseError):
                IdaDataBase(dsn = 'NOTEXISTING_DATASOURCE')

        if idadb._con_type == 'jdbc':
            with pytest.raises(IdaDataBaseError):
                IdaDataBase(dsn='jdbc:NOTEXISTING_DATASOURCE')
            with pytest.raises(IdaDataBaseError):
                IdaDataBase(dsn = 'jdbc:db2://awh-yp-small03.services.dal.bluemix.net:50000/BLUDB:user=XXXXXXXXXX;password=XXXXXXXXXXXX',
                        uid="hello", pwd="world")
            with pytest.raises(IdaDataBaseError):
                IdaDataBase(dsn = 'jdbc:db2://awh-yp-small03.services.dal.bluemix.net:50000/BLUDB')


class Test_ConnexionManagement(object):

    def test_idadb_commit(self, idadb, df):
        try : idadb.drop_table("TEST_COMMIT_5869703824607040")
        except : pass
        idadb._create_table(df, "TEST_COMMIT_5869703824607040")
        idadb.commit()
        #idadb.rback()
        assert(idadb.exists_table("TEST_COMMIT_5869703824607040") == 1)
        idadb.drop_table("TEST_COMMIT_5869703824607040")
        idadb.commit()

    def test_idadb_rollback(self, idadb, df):
        # TODO: Does not work for JDBC -> Why no rollback for jaydebeapi ?
        if idadb._con_type == 'odbc':
            try:
                idadb.drop_table("TEST_ROLLBACK_59673030586849305074")
            except:
                pass
            idadb.commit()
            idadb._create_table(df, "TEST_ROLLBACK_59673030586849305074")
            idadb.rollback()
            idadb.commit()
            assert(idadb.exists_table("TEST_ROLLBACK_59673030586849305074") == 0)

    def test_idadb_close(self, idadb_tmp):
        idadb_tmp.close()
        with pytest.raises(IdaDataBaseError):
            idadb_tmp.show_tables()

    def test_reconnect(self, idadb_tmp):
        idadb_tmp.close()
        with pytest.raises(IdaDataBaseError):
            idadb_tmp.show_tables()
        idadb_tmp.reconnect()
        assert(isinstance(idadb_tmp.show_tables(), pandas.DataFrame))


class Test_UploadDataFrame(object):

    def test_idadb_as_idadataframe(self, idadb, df):
        ida = idadb.as_idadataframe(df, "TEST_AS_IDADF_18729493954_23849590", clear_existing = True)
        assert(all(ida.columns == df.columns))
        assert(list(ida.index) == list(df.index))
        assert(ida.shape == df.shape)
        idadb.drop_table("TEST_AS_IDADF_18729493954_23849590")

    # test schema support
    def test_idadb_as_idadataframe_with_schema(self, idadb, df):
        ida = idadb.as_idadataframe(df, "DUMMY.TEST_AS_IDADF_18729493954_23849590", clear_existing = True)
        assert(all(ida.columns == df.columns))
        assert(list(ida.index) == list(df.index))
        assert(ida.shape == df.shape)
        idadb.drop_table("DUMMY.TEST_AS_IDADF_18729493954_23849590")

    # test if the columns function returns the correct values
    # if two tables exist with the same name but different schema
    def test_idadb_as_idadataframe_same_tablename(self, idadb, df):
        ida11 = idadb.as_idadataframe(df, "TEST_AS_IDADF_18729493954_23849590", clear_existing = True)
        ida21 = idadb.as_idadataframe(df, "DUMMY.TEST_AS_IDADF_18729493954_23849590", clear_existing = True)
        assert(all(ida11.columns == df.columns))
        assert(all(ida21.columns == df.columns))
        ida12 = IdaDataFrame(idadb, "TEST_AS_IDADF_18729493954_23849590")
        ida22 = IdaDataFrame(idadb, "DUMMY.TEST_AS_IDADF_18729493954_23849590")
        assert(all(ida12.columns == df.columns))
        assert(all(ida22.columns == df.columns))
        idadb.drop_table("TEST_AS_IDADF_18729493954_23849590")
        idadb.drop_table("DUMMY.TEST_AS_IDADF_18729493954_23849590")

    def test_idadb_ida_query(self, idadb, idadf):
        query = "SELECT * FROM %s FETCH LIMIT 5"%idadf.name
        df = idadb.ida_query(query)
        assert(isinstance(df,pandas.DataFrame))
        assert(len(idadf.columns) == len(df.columns))
        assert(len(df) == 5)

    def test_idadb_ida_query_first_row_only(self, idadb, idadf, df):
        query = "SELECT * FROM %s FETCH LIMIT 5"%idadf.name
        downloaded_df = idadb.ida_query(query, first_row_only=True)
        assert(isinstance(downloaded_df,tuple))
        assert(len(downloaded_df) == len(df.loc[0]))

    def test_idadb_ida_scalar_query(self, idadb, idadf):
        query = "SELECT * FROM %s FETCH LIMIT 5"%idadf.name
        downloaded_df = idadb.ida_scalar_query(query)
        assert(isinstance(downloaded_df,six.string_types)|isinstance(downloaded_df,Number))


    ################# MORE TEST TO DO HERE



# -*- coding: utf-8 -*-
"""
Created on Tue Dec  1 12:29:30 2015

@author: efouche
"""
from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
from future import standard_library
standard_library.install_aliases()


from collections import OrderedDict

import pandas as pd
import numpy as np
import six

from nzpyida.internals import idadf_state
from nzpyida.utils import timed
from nzpyida.feature_selection.private import _check_input

@idadf_state
@timed
def gini_pairwise(idadf, target=None, features=None, ignore_indexer=True):
    """
    Compute the conditional gini coefficients between a set of features and a 
    set of target in an IdaDataFrame. 
    
    Parameters
    ----------
    idadf : IdaDataFrame
    
    target : str or list of str, optional
        A column or list of columns against to be used as target. Per default, 
        consider all columns
    
    features : str or list of str, optional
        A column or list of columns to be used as features. Per default, 
        consider all columns. 
    
    ignore_indexer : bool, default: True
        Per default, ignore the column declared as indexer in idadf
        
    Returns
    -------
    Pandas.DataFrame or Pandas.Series if only one target
    
    Notes
    -----
    Input columns as target and features should be categorical, otherwise 
    this measure does not make much sense. 
    
    Examples
    --------
    >>> idadf = IdaDataFrame(idadb, "IRIS")
    >>> gini_pairwise(idadf)
    """
    # Check input
    target, features = _check_input(idadf, target, features, ignore_indexer)
        
    gini_dict = OrderedDict()
    length = len(idadf)

    if idadf._idadb._is_netezza_system():
        power_function = "POW"
        div_term = "* POW(c,-1)) * POW(%s,-1)"%length
    else:
        power_function = "POWER"
        div_term = "/c)/%s"%length

    for t in target:
        gini_dict[t] = OrderedDict() 
        features_notarget = [x for x in features if (x != t)]
        
        for feature in features_notarget:
            if t not in gini_dict:
                gini_dict[t] = OrderedDict()
            
            query = ("SELECT SUM((%s(c,2) - gini)%s FROM "+
            "(SELECT SUM(%s(count,2)) as gini, SUM(count) as c FROM "+
            "(SELECT CAST(COUNT(*) AS FLOAT) AS count, \"%s\" FROM %s GROUP BY \"%s\",\"%s\") AS T1 "+
            "GROUP BY \"%s\") AS T2 ")
            query0 = query%(power_function, div_term, power_function, feature, idadf.name, t, feature, feature)
            gini_dict[t][feature] = idadf.ida_scalar_query(query0)
            
    result = pd.DataFrame(gini_dict).fillna(np.nan)
        
    if len(result.columns) > 1:
        order = [x for x in result.columns if x in features] + [x for x in features if x not in result.columns]
        result = result.reindex(order)
       
    result = result.dropna(axis=1, how="all")
    
    if len(result.columns) == 1:
        if len(result) == 1:
            result = result.iloc[0,0]
        else:
            result = result[result.columns[0]].copy()
            result.sort_values(ascending = True)
    else:
        result = result.fillna(0)
    
    return result
               
    
        
    
    
    
@idadf_state
@timed
def gini(idadf, features=None, ignore_indexer=True):
    """
    Compute the gini coefficients for a set of features in an IdaDataFrame. 
    
    Parameters
    ----------
    idadf : IdaDataFrame
    
    features : str or list of str, optional
        A column or list of columns to be used as features. Per default, 
        consider all columns. 
    
    ignore_indexer : bool, default: True
        Per default, ignore the column declared as indexer in idadf
        
    Returns
    -------
    Pandas.Series
    
    Notes
    -----
    Input column should be categorical, otherwise this measure does not make 
    much sense. 
    
    Examples
    --------
    >>> idadf = IdaDataFrame(idadb, "IRIS")
    >>> gini(idadf)
    """
    if features is None:
        features = list(idadf.columns)
    else:
        if isinstance(features, six.string_types):
            features = [features]

    if ignore_indexer is True:
        if idadf.indexer:
            if idadf.indexer in features:
                features.remove(idadf.indexer)
      
        
    value_dict = OrderedDict()
        
    length = len(idadf)**2

    if idadf._idadb._is_netezza_system():
      power_function = "POW"
      div_term = "* POW(%s, -1)"%length
    else:
      power_function = "POWER"
      div_term ="/%s"%length

    for feature in features:
        
        subquery = "SELECT COUNT(*) AS count FROM %s GROUP BY \"%s\""%(idadf.name, feature)
        query = "SELECT (%s - SUM(%s(count,2)))%s FROM (%s) AS T "%(length, power_function, div_term, subquery)
        value_dict[feature] = idadf.ida_scalar_query(query)
            
        if len(features) > 1:
            result = pd.Series(value_dict) 
        else:
            result = value_dict[feature]
    
    return result


# -*- coding: utf-8 -*-
"""
Created on Mon Nov 23 09:05:39 2015

@author: efouche
"""
from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
from future import standard_library
standard_library.install_aliases()
from nzpyida.internals import idadf_state
from nzpyida.utils import timed
from collections import OrderedDict

import pandas as pd

import six

@idadf_state
def entropy(idadf, target=None, mode="normal", execute=True, ignore_indexer=True):
    """
    Compute the entropy for a set of features in an IdaDataFrame. 
    
    Parameters
    ----------
    idadf: IdaDataFrame
    
    target: str or list of str, optional
        A column or list of columns to be used as features. Per default, 
        consider all columns. 
    
    mode: "normal" or "raw"
        Experimental
        
    execute: bool, default:True
        Experimental. Execute the request or return the correponding SQL query 
    
    ignore_indexer: bool, default: True
        Per default, ignore the column declared as indexer in idadf
        
    Returns
    -------
    Pandas.Series
    
    Notes
    -----
    Input column should be categorical, otherwise this measure does not make 
    much sense. 
    
    Examples
    --------
    >>> idadf = IdaDataFrame(idadb, "IRIS")
    >>> entropy(idadf)
    """
    if target is not None:
        if isinstance(target, six.string_types):
            target = [target]
            
        targetstr = "\",\"".join(target)
        subquery = "SELECT COUNT(*) AS a FROM %s GROUP BY \"%s\""%(idadf.name,targetstr)
        if mode == "normal":
            length = len(idadf)
            query = "SELECT (SUM(-a*LOG(a))/%s+LOG(%s))/LOG(2) FROM (%s) AS T"%(length, length, subquery)
        elif mode == "raw":
            query = "SELECT SUM(-a*LOG(a)) FROM(%s) AS T "%(subquery)
        
        if not execute:
            query = query[:query.find("FROM")] + ",'%s'"%"\',\'".join(target) + query[query.find("FROM"):]
            return query
        return idadf.ida_scalar_query(query)
    else:
        entropy_dict = OrderedDict()
        columns = list(idadf.columns)
        # Remove indexer
        if ignore_indexer:
            if idadf.indexer:
                if idadf.indexer in columns:
                    columns.remove(idadf.indexer)
                    
        for column in columns:
           entropy_dict[column] = entropy(idadf, column, mode = mode)
                    
        # Output
        if len(columns) > 1:
            result = pd.Series(entropy_dict)
            result.sort_values(ascending = False)
        else:
            result = entropy_dict[columns[0]]
        return result
    
def entropy_stats(idadf, target=None, mode="normal", execute = True, ignore_indexer=True):
    """
    Similar to ibmdbby.feature_selection.entropy.entrop but use DB2 statistics
    to speed the computation. Returns an approximate value. Experimental. 
    
    Parameters
    ----------
    idadf: IdaDataFrame
    
    target: str or list of str, optional
        A column or list of columns to be used as features. Per default, 
        consider all columns. 
    
    mode: "normal" or "raw"
        Experimental
        
    execute: bool, default:True
        Experimental. Execute the request or return the correponding SQL query 
    
    ignore_indexer: bool, default: True
        Per default, ignore the column declared as indexer in idadf
        
    Returns
    -------
    Pandas.Series
    
    Notes
    -----
    Input column should be categorical, otherwise this measure does not make 
    much sense. 
    
    Cannot handle columns that are not physically existing in the database, 
    since no statistics are available for them. 
    
    Examples
    --------
    >>> idadf = IdaDataFrame(idadb, "IRIS")
    >>> entropy_stats(idadf)
    """
    if target is not None:
        subquery = "SELECT VALCOUNT as a FROM SYSCAT.COLDIST WHERE TABSCHEMA='%s' AND TABNAME = '%s' AND COLNAME='%s' AND TYPE='F' AND COLVALUE IS NOT NULL"%(idadf.schema, idadf.tablename, target)
        if mode == "normal":
            query = "SELECT(SUM(-a*LOG(a))/SUM(a)+LOG(SUM(a)))/LOG(2)FROM(%s)"%(subquery)
        elif mode == "raw":
            query = "SELECT SUM(-a*LOG(a)) FROM(%s)"%(subquery)
        
        if not execute:
            return query
        return idadf.ida_scalar_query(query)
    else:
        entropy_dict = OrderedDict()
        columns = list(idadf.columns)
        # Remove indexer
        if ignore_indexer:
            if idadf.indexer:
                if idadf.indexer in columns:
                    columns.remove(idadf.indexer)
                    
        for column in columns:
           entropy_dict[column] = entropy_stats(idadf, column, mode = mode)
                    
        # Output
        if len(columns) > 1:
            result = pd.Series(entropy_dict)
            result.sort_values(ascending = False)
        else:
            result = entropy_dict[columns[0]]
        return result


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#-----------------------------------------------------------------------------
# Copyright (c) 2015, IBM Corp.
# All rights reserved.
#
# Distributed under the terms of the BSD Simplified License.
#
# The full license is in the LICENSE file, distributed with this software.
#-----------------------------------------------------------------------------

"""
Test module for indexing of IdaDataFrameObjects
"""
from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
from future import standard_library
standard_library.install_aliases()

import nzpyida

class Test_projection(object):

    def test_projection_1_col_idadf(self, idadf):
        col = idadf.columns[0]
        ida = idadf[[col]]
        assert(type(ida) == nzpyida.frame.IdaDataFrame)
        assert(len(ida.columns) == 1)
        head = ida.head()
        assert (head.name == idadf.columns[0])
        pass

    def test_projection_1_col_idaseries(self, idadf):
        col = idadf.columns[0]
        ida = idadf[col]
        assert(type(ida) == nzpyida.series.IdaSeries)
        assert(len(ida.columns) == 1)
        assert(ida.column == idadf.columns[0])
        head = ida.head()
        assert(head.name == idadf.columns[0])
        pass

    def test_projection_1_col_loc(idadf):
        pass

    def test_projection_list_of_col(idadf):
        pass

    def test_projection_list_of_col_loc(idadf):
        pass

    def test_projection_error(idadf):
        pass


class Test_selection(object):

    def test_selection_1_row(idadf):
        pass

    def test_selection_list_of_rows(idadf):
        pass

    def test_selection_slice_of_rows(idadf):
        pass

    def test_selection_slice_error(idadf):
        pass


class Test_projection_selection(object):

    def test_selection_1_row_projection_1_col_idadf(idadf):
        pass

    def test_selection_1_row_projection_1_col_idaseries(idadf):
        pass

    def test_selection_1_row_projection_list_of_col(idadf):
        pass

    def test_selection_list_of_rows_projection_1_col_idadf(idadf):
        pass

    def test_selection_list_of_rows_projection_1_col_idaseries(idadf):
        pass

    def test_selection_list_of_rows_projection_list_of_col(idadf):
        pass

    def test_selection_slice_of_rows_projection_1_col_idadf(idadf):
        pass

    def test_selection_slice_of_rows_projection_1_col_idaseries(idadf):
        pass

    def test_selection_slice_of_rows_projection_list_of_col(idadf):
        pass

    def test_selection_projection_error(idadf):
        pass

class Test_Loc(object):

    def test_loc_error(idadf):
        pass


# -*- coding: utf-8 -*-
"""
Created on Mon Nov 23 08:59:00 2015

@author: efouche
"""
from __future__ import division
from __future__ import unicode_literals
from __future__ import print_function
from __future__ import absolute_import
from future import standard_library
standard_library.install_aliases()

from collections import OrderedDict

from nzpyida.internals import idadf_state
from nzpyida.utils import timed

import pandas as pd
from numpy import log
import numpy as np

from nzpyida.feature_selection.entropy import entropy

from nzpyida.feature_selection.private import _check_input

@idadf_state
@timed
def info_gain(idadf, target = None, features = None, ignore_indexer=True):
    """
    Compute the information gain / mutual information coefficients between a 
    set of features and a set of target in an IdaDataFrame. 
    
    Parameters
    ----------
    idadf : IdaDataFrame
    
    target : str or list of str, optional
        A column or list of columns against to be used as target. Per default, 
        consider all columns
    
    features : str or list of str, optional
        A column or list of columns to be used as features. Per default, 
        consider all columns. 
    
    ignore_indexer : bool, default: True
        Per default, ignore the column declared as indexer in idadf
        
    Returns
    -------
    Pandas.DataFrame or Pandas.Series if only one target
    
    Notes
    -----
    Input columns as target and features should be categorical, otherwise 
    this measure does not make much sense. 
    
    Examples
    --------
    >>> idadf = IdaDataFrame(idadb, "IRIS")
    >>> info_gain(idadf)
    """
    # Check input
    target, features = _check_input(idadf, target, features, ignore_indexer)
    
    entropy_dict = OrderedDict()
    length = len(idadf)
    loglength = log(length)
    
    values = OrderedDict()
    
    for t in target:
        if t not in values:
            values[t] = OrderedDict() 
        features_notarget = [x for x in features if (x != t)]
        
        for feature in features_notarget:
            if feature not in values:
                values[feature] = OrderedDict()
            if t not in values[feature]:
                if t not in entropy_dict:
                    entropy_dict[t] = entropy(idadf, t, mode = "raw")
                if feature not in entropy_dict:
                    entropy_dict[feature] = entropy(idadf, feature, mode = "raw")
                join_entropy = entropy(idadf, [t] + [feature], mode = "raw")            
                
                value = ((entropy_dict[t] + entropy_dict[feature] - join_entropy)/length + loglength)/log(2)
                values[t][feature] = value
                if feature in target:
                    values[feature][t] = value
    
    result = pd.DataFrame(values).fillna(np.nan)
    result = result.dropna(axis=1, how="all")
    
    if len(result.columns) > 1:
        order = [x for x in result.columns if x in features] + [x for x in features if x not in result.columns]
        result = result.reindex(order)
    
    if len(result.columns) == 1:
        if len(result) == 1:
            result = result.iloc[0,0]
        else:
            result = result[result.columns[0]].copy()
            result.sort_values(inplace=True, ascending=False)

    return result        




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#-----------------------------------------------------------------------------
# Copyright (c) 2015, IBM Corp.
# All rights reserved.
#
# Distributed under the terms of the BSD Simplified License.
#
# The full license is in the LICENSE file, distributed with this software.
#-----------------------------------------------------------------------------

"""
Test module for IdaDataFrameObjects
"""
from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
from builtins import round
from builtins import zip
from future import standard_library
standard_library.install_aliases()

import pandas
import pytest

from nzpyida.feature_selection import pearson
from nzpyida.feature_selection import spearman
from nzpyida.feature_selection import ttest
from nzpyida.feature_selection import chisquared
from nzpyida.feature_selection import gini, gini_pairwise
from nzpyida.feature_selection import entropy
from nzpyida.feature_selection import info_gain, gain_ratio, su

# Test symmetry

class Test_PearsonCorrelation(object):

    def test_pearson_default(self, idadf):
        data = idadf._table_def() 
        columns = list(data.loc[data['VALTYPE'] == "NUMERIC"].index)
        if len(columns) > 1:
            result = pearson(idadf, features = columns)
            assert(isinstance(result, pandas.core.frame.DataFrame))
            assert(len(result.columns) == len(columns))
            assert(len(result.index) == len(columns))
            result2 = pearson(idadf)
            assert(all(result == result2))
            result = result.fillna(0) # np.nan values are not equal when compared
            assert(all(result == result.T)) # symmetry
                
    def test_pearson_valueError(self, idadf):
        if len(idadf.columns) > 0:
            with pytest.raises(ValueError):
                pearson(idadf, features = idadf.columns[0])
            with pytest.raises(ValueError):
                pearson(idadf, target= idadf.columns[0], features = idadf.columns[0])
                
    def test_pearson_TypeError(self, idadf):
        data = idadf._table_def() 
        columns = list(data.loc[data['VALTYPE'] != "NUMERIC"].index)
        if len(columns) > 1:
            with pytest.raises(TypeError):
                pearson(idadf, features = columns)
                
    def test_pearson_one_target(self, idadf):
        data = idadf._table_def() 
        columns = list(data.loc[data['VALTYPE'] == "NUMERIC"].index)
        if len(columns) > 1:
            result = pearson(idadf, target = columns[0])
            assert(isinstance(result, pandas.core.series.Series))
            assert(len(result) == (len(columns)-1))
            
    def test_pearson_multiple_target(self, idadf):
        data = idadf._table_def() 
        columns = list(data.loc[data['VALTYPE'] == "NUMERIC"].index)
        if len(columns) > 1:
            result = pearson(idadf, target = [columns[0],columns[1]])
            assert(isinstance(result, pandas.core.frame.DataFrame))
            assert(len(result.columns) == 2)
            assert(len(result.index) == len(columns))
            
    def test_pearson_one_target_one_feature(self, idadf):
        data = idadf._table_def() 
        columns = list(data.loc[data['VALTYPE'] == "NUMERIC"].index)
        if len(columns) > 1:
            result = pearson(idadf, target = columns[0], features=[columns[1]])
            assert(isinstance(result, float))
            result2 = pearson(idadf, target = columns[1], features=[columns[0]])
            assert(round(result,3) == round(result2,3)) # symmetry
    
class Test_SpearmanRankCorrelation(object):

    def test_spearman_default(self, idadf):
        data = idadf._table_def() 
        columns = list(data.loc[data['VALTYPE'] == "NUMERIC"].index)
        if len(columns) > 1:
            result = spearman(idadf, features = columns)
            assert(isinstance(result, pandas.core.frame.DataFrame))
            assert(len(result.columns) == len(columns))
            assert(len(result.index) == len(columns))
            result2 = spearman(idadf)
            assert(all(result == result2))
            result = result.fillna(0) # np.nan values are not equal when compared
            assert(all(result == result.T)) # symmetry
                
    def test_spearman_valueError(self, idadf):
        if len(idadf.columns) > 0:
            with pytest.raises(ValueError):
                spearman(idadf, features = idadf.columns[0])
            with pytest.raises(ValueError):
                spearman(idadf, target= idadf.columns[0], features = idadf.columns[0])
                
    def test_spearman_TypeError(self, idadf):
        data = idadf._table_def() 
        columns = list(data.loc[data['VALTYPE'] != "NUMERIC"].index)
        if len(columns) > 1:
            with pytest.raises(TypeError):
                spearman(idadf, features = columns)
                
    def test_spearman_one_target(self, idadf):
        data = idadf._table_def() 
        columns = list(data.loc[data['VALTYPE'] == "NUMERIC"].index)
        if len(columns) > 1:
            result = spearman(idadf, target = columns[0])
            assert(isinstance(result, pandas.core.series.Series))
            assert(len(result) == (len(columns)-1))
            
    def test_spearman_multiple_target(self, idadf):
        data = idadf._table_def() 
        columns = list(data.loc[data['VALTYPE'] == "NUMERIC"].index)
        if len(columns) > 1:
            result = spearman(idadf, target = [columns[0],columns[1]])
            assert(isinstance(result, pandas.core.frame.DataFrame))
            assert(len(result.columns) == 2)
            assert(len(result.index) == len(columns))
            
    def test_spearman_one_target_one_feature(self, idadf):
        data = idadf._table_def() 
        columns = list(data.loc[data['VALTYPE'] == "NUMERIC"].index)
        if len(columns) > 1:
            result = spearman(idadf, target = columns[0], features=[columns[1]])
            assert(isinstance(result, float))
            result2 = spearman(idadf, target = columns[1], features=[columns[0]])
            assert(round(result,3) == round(result2,3)) # symmetry
    
class Test_Tstatistics(object):

    def test_ttest_default(self, idadf):
        data = idadf._table_def() 
        columns = list(data.loc[data['VALTYPE'] == "NUMERIC"].index)
        if len(columns) > 1:
            result = ttest(idadf, features = columns)
            assert(isinstance(result, pandas.core.frame.DataFrame))
            assert(len(result.columns) == len(columns))
            assert(len(result.index) == len(columns))
            result = result.fillna(0) # np.nan values are not equal when compared
            result = result[result.index]
            assert(all(result != result.T)) # asymmetry
            
    def test_ttest_valueError(self, idadf):
        data = idadf._table_def() 
        columns = list(data.loc[data['VALTYPE'] != "NUMERIC"].index)
        if len(columns) > 0: # Raise no numerical features
            with pytest.raises(ValueError):
                ttest(idadf, features = columns)
        if len(idadf.columns) > 0: 
            with pytest.raises(ValueError): # Cannot compute correlation coefficients of only one column (...), need at least 2
                ttest(idadf, features = idadf.columns[0])
            with pytest.raises(ValueError): # The correlation value of two same columns is always maximal
                ttest(idadf, target= idadf.columns[0], features = idadf.columns[0])
                
    def test_ttest_TypeError(self, idadf):
        data = idadf._table_def() 
        columns = list(data.loc[data['VALTYPE'] != "NUMERIC"].index)
        if len(columns) > 1:
            with pytest.raises(TypeError):
                ttest(idadf, features = columns)
                
    def test_ttest_one_target(self, idadf):
        data = idadf._table_def() 
        columns = list(data.loc[data['VALTYPE'] == "NUMERIC"].index)
        if len(columns) > 1:
            result = ttest(idadf, target = columns[0])
            assert(isinstance(result, pandas.core.series.Series))
            assert(len(result) == (len(columns)-1))
            
    def test_ttest_multiple_target(self, idadf):
        data = idadf._table_def() 
        columns = list(data.loc[data['VALTYPE'] == "NUMERIC"].index)
        if len(columns) > 1:
            result = ttest(idadf, target = [columns[0],columns[1]])
            assert(isinstance(result, pandas.core.frame.DataFrame))
            assert(len(result.columns) == 2)
            assert(len(result.index) == len(columns))
            
    def test_ttest_one_target_one_feature(self, idadf):
        data = idadf._table_def() 
        columns = list(data.loc[data['VALTYPE'] == "NUMERIC"].index)
        if len(columns) > 1:
            result = ttest(idadf, target = columns[0], features=[columns[1]])
            assert(isinstance(result, float))
            
    
class Test_Chisquared(object):
    
    def test_chisquared_default(self, idadf):
        if len(idadf.columns) > 1:
            result = chisquared(idadf, features = idadf.columns)
            assert(isinstance(result, pandas.core.frame.DataFrame))
            assert(len(result.columns) == len(idadf.columns))
            assert(len(result.index) == len(idadf.columns))
            result2 = chisquared(idadf)
            assert(all(result == result2))
            result = result.fillna(0) # np.nan values are not equal when compared
            assert(all(result == result.T)) # symmetry
            
    def test_chisquared_valueError(self, idadf):
        if len(idadf.columns) > 0: 
            with pytest.raises(ValueError): # Cannot compute correlation coefficients of only one column (...), need at least 2
                chisquared(idadf, features = idadf.columns[0])
            with pytest.raises(ValueError): # The correlation value of two same columns is always maximal
                chisquared(idadf, target= idadf.columns[0], features = idadf.columns[0])
                
    def test_chisquared_one_target(self, idadf):
        if len(idadf.columns) > 1:
            result = chisquared(idadf, target = idadf.columns[0])
            assert(isinstance(result, pandas.core.series.Series))
            assert(len(result) == (len(idadf.columns)-1))
            
    def test_chisquared_multiple_target(self, idadf):
        if len(idadf.columns) > 1:
            result = chisquared(idadf, target = [idadf.columns[0],idadf.columns[1]])
            assert(isinstance(result, pandas.core.frame.DataFrame))
            assert(len(result.columns) == 2)
            assert(len(result.index) == len(idadf.columns))
            
    def test_chisquared_one_target_one_feature(self, idadf):
        if len(idadf.columns) > 1:
            result = chisquared(idadf, target = idadf.columns[0], features=[idadf.columns[1]])
            assert(isinstance(result, float))
            result2 = chisquared(idadf, target = idadf.columns[1], features=[idadf.columns[0]])
            assert(round(result,3) == round(result2,3)) # symmetry
    
class Test_Gini_Index(object):

    def test_gini_default(self, idadf):
        if len(idadf.columns) > 1:
            result = gini(idadf, features = idadf.columns)
            assert(isinstance(result, pandas.core.series.Series))
            assert(len(result.index) == len(idadf.columns))
            result2 = gini(idadf)
            assert(all(result == result2))
            
    def test_gini_multiple_columns(self, idadf):
        if len(idadf.columns) > 1:
            result = gini(idadf, features = [idadf.columns[0],idadf.columns[1]])
            assert(isinstance(result, pandas.core.series.Series))
            assert(len(result) == 2)
            
    def test_gini_one_column(self, idadf):
        if len(idadf.columns) >= 1:
            result = gini(idadf, features = idadf.columns[0])
            assert(isinstance(result, float))
            
class Test_Gini_Pairwise(object):

    def test_gini_pairwise_default(self, idadf):
        if len(idadf.columns) > 1:
            result = gini_pairwise(idadf, features = idadf.columns)
            assert(isinstance(result, pandas.core.frame.DataFrame))
            assert(len(result.columns) == len(idadf.columns))
            assert(len(result.index) == len(idadf.columns))
            result2 = gini_pairwise(idadf)
            assert(all(result == result2)) 
            result = result.fillna(0) # np.nan values are not equal when compared
            assert(any(result != result.T)) # asymmetry
            
    def test_gini_pairwise_valueError(self, idadf):
        if len(idadf.columns) > 0: 
            with pytest.raises(ValueError): # Cannot compute correlation coefficients of only one column (...), need at least 2
                gini_pairwise(idadf, features = idadf.columns[0])
            with pytest.raises(ValueError): # The correlation value of two same columns is always maximal
                gini_pairwise(idadf, target= idadf.columns[0], features = idadf.columns[0])
                
    def test_gini_pairwise_one_target(self, idadf):
        if len(idadf.columns) > 1:
            result = gini_pairwise(idadf, target = idadf.columns[0])
            assert(isinstance(result, pandas.core.series.Series))
            assert(len(result) == (len(idadf.columns)-1))
            
    def test_gini_pairwise_multiple_target(self, idadf):
        if len(idadf.columns) > 1:
            result = gini_pairwise(idadf, target = [idadf.columns[0],idadf.columns[1]])
            assert(isinstance(result, pandas.core.frame.DataFrame))
            assert(len(result.columns) == 2)
            assert(len(result.index) == len(idadf.columns))
            
    def test_gini_pairwise_one_target_one_feature(self, idadf):
        if len(idadf.columns) > 1:
            result = gini_pairwise(idadf, target = idadf.columns[0], features=[idadf.columns[1]])
            assert(isinstance(result, float))
    
class Test_Entropy(object):

    def test_entropy_default(self, idadf):
        if len(idadf.columns) > 1:
            result = entropy(idadf)
            assert(isinstance(result, pandas.core.series.Series))
            assert(len(result.index) == len(idadf.columns))
            
    def test_entropy_multiple_columns(self, idadf):
        if len(idadf.columns) > 1:
            result = entropy(idadf, target = [idadf.columns[0],idadf.columns[1]])
            assert(isinstance(result, float))
            
    def test_entropy_one_column(self, idadf):
        if len(idadf.columns) >= 1:
            result = entropy(idadf, target = idadf.columns[0])
            assert(isinstance(result, float))
    
class Test_Information_Gain(object):

    def test_info_gain_default(self, idadf):
        if len(idadf.columns) > 1:
            result = info_gain(idadf, features = idadf.columns)
            assert(isinstance(result, pandas.core.frame.DataFrame))
            assert(len(result.columns) == len(idadf.columns))
            assert(len(result.index) == len(idadf.columns))
            result2 = info_gain(idadf)
            assert(all(result == result2))
            result = result.fillna(0) # np.nan values are not equal when compared
            assert(all(result == result.T)) # symmetry
            
    def test_info_gain_valueError(self, idadf):
        if len(idadf.columns) > 0: 
            with pytest.raises(ValueError): # Cannot compute correlation coefficients of only one column (...), need at least 2
                info_gain(idadf, features = idadf.columns[0])
            with pytest.raises(ValueError): # The correlation value of two same columns is always maximal
                info_gain(idadf, target= idadf.columns[0], features = idadf.columns[0])
                
    def test_info_gain_one_target(self, idadf):
        if len(idadf.columns) > 1:
            result = info_gain(idadf, target = idadf.columns[0])
            assert(isinstance(result, pandas.core.series.Series))
            assert(len(result) == (len(idadf.columns)-1))
            
    def test_info_gain_multiple_target(self, idadf):
        if len(idadf.columns) > 1:
            result = info_gain(idadf, target = [idadf.columns[0],idadf.columns[1]])
            assert(isinstance(result, pandas.core.frame.DataFrame))
            assert(len(result.columns) == 2)
            assert(len(result.index) == len(idadf.columns))
            
    def test_info_gain_one_target_one_feature(self, idadf):
        if len(idadf.columns) > 1:
            result = info_gain(idadf, target = idadf.columns[0], features=[idadf.columns[1]])
            assert(isinstance(result, float))
            result2 = info_gain(idadf, target = idadf.columns[1], features=[idadf.columns[0]])
            assert(round(result,3) == round(result2,3)) # symmetry
    
class Test_Symmetric_Gain_Ratio(object):

    def test_sym_gain_ratio_default(self, idadf):
        if len(idadf.columns) > 1:
            result = gain_ratio(idadf, features = idadf.columns, symmetry=True)
            assert(isinstance(result, pandas.core.frame.DataFrame))
            assert(len(result.columns) == len(idadf.columns))
            assert(len(result.index) == len(idadf.columns))
            result2 = gain_ratio(idadf, symmetry=True)
            assert(all(result == result2))
            result = result.fillna(0) # np.nan values are not equal when compared
            assert(all(result == result.T)) # symmetry
            
    def test_sym_gain_ratio_valueError(self, idadf):
        if len(idadf.columns) > 0: 
            with pytest.raises(ValueError): # Cannot compute correlation coefficients of only one column (...), need at least 2
                gain_ratio(idadf, features = idadf.columns[0], symmetry=True)
            with pytest.raises(ValueError): # The correlation value of two same columns is always maximal
                gain_ratio(idadf, target= idadf.columns[0], features = idadf.columns[0], symmetry=True)
                
    def test_sym_gain_ratio_one_target(self, idadf):
        if len(idadf.columns) > 1:
            result = gain_ratio(idadf, target = idadf.columns[0], symmetry=True)
            assert(isinstance(result, pandas.core.series.Series))
            assert(len(result) == (len(idadf.columns)-1))
            
    def test_sym_gain_ratio_multiple_target(self, idadf):
        if len(idadf.columns) > 1:
            result = gain_ratio(idadf, target = [idadf.columns[0],idadf.columns[1]], symmetry=True)
            assert(isinstance(result, pandas.core.frame.DataFrame))
            assert(len(result.columns) == 2)
            assert(len(result.index) == len(idadf.columns))
            
    def test_sym_gain_ratio_one_target_one_feature(self, idadf):
        if len(idadf.columns) > 1:
            result = gain_ratio(idadf, target = idadf.columns[0], features=[idadf.columns[1]], symmetry=True)
            assert(isinstance(result, float))
            result2 = gain_ratio(idadf, target = idadf.columns[1], features=[idadf.columns[0]], symmetry=True)
            assert(round(result,3) == round(result2,3)) # symmetry
    
class Test_Asymmetric_Gain_Ratio(object):

    def test_asym_gain_ratio_default(self, idadf):
        if len(idadf.columns) > 1:
            result = gain_ratio(idadf, features = idadf.columns, symmetry=False)
            assert(isinstance(result, pandas.core.frame.DataFrame))
            assert(len(result.columns) == len(idadf.columns))
            assert(len(result.index) == len(idadf.columns))
            result2 = gain_ratio(idadf, symmetry=False)
            assert(all(result == result2))
            result = result.fillna(0) # np.nan values are not equal when compared
            assert(any(result != result.T)) # asymmetry
            
    def test_asym_gain_ratio_valueError(self, idadf):
        if len(idadf.columns) > 0: 
            with pytest.raises(ValueError): # Cannot compute correlation coefficients of only one column (...), need at least 2
                gain_ratio(idadf, features = idadf.columns[0], symmetry=False)
            with pytest.raises(ValueError): # The correlation value of two same columns is always maximal
                gain_ratio(idadf, target= idadf.columns[0], features = idadf.columns[0], symmetry=False)
                
    def test_asym_gain_ratio_one_target(self, idadf):
        if len(idadf.columns) > 1:
            result = gain_ratio(idadf, target = idadf.columns[0], symmetry=False)
            assert(isinstance(result, pandas.core.series.Series))
            assert(len(result) == (len(idadf.columns)-1))
            
    def test_asym_gain_ratio_multiple_target(self, idadf):
        if len(idadf.columns) > 1:
            result = gain_ratio(idadf, target = [idadf.columns[0],idadf.columns[1]], symmetry=False)
            assert(isinstance(result, pandas.core.frame.DataFrame))
            assert(len(result.columns) == 2)
            assert(len(result.index) == len(idadf.columns))
            
    def test_asym_gain_ratio_one_target_one_feature(self, idadf):
        if len(idadf.columns) > 1:
            result = gain_ratio(idadf, target = idadf.columns[0], features=[idadf.columns[1]], symmetry=False)
            assert(isinstance(result, float))
            
class Test_Symmetric_Uncertainty(object):

    def test_su_default(self, idadf):
        if len(idadf.columns) > 1:
            result = su(idadf, features = idadf.columns)
            assert(isinstance(result, pandas.core.frame.DataFrame))
            assert(len(result.columns) == len(idadf.columns))
            assert(len(result.index) == len(idadf.columns))
            result2 = su(idadf)
            assert(all(result == result2))
            result = result.fillna(0) # np.nan values are not equal when compared
            assert(all(result == result.T)) # symmetry
            
    def test_su_valueError(self, idadf):
        if len(idadf.columns) > 0: 
            with pytest.raises(ValueError): # Cannot compute correlation coefficients of only one column (...), need at least 2
                su(idadf, features = idadf.columns[0])
            with pytest.raises(ValueError): # The correlation value of two same columns is always maximal
                su(idadf, target= idadf.columns[0], features = idadf.columns[0])
                
    def test_su_one_target(self, idadf):
        if len(idadf.columns) > 1:
            result = su(idadf, target = idadf.columns[0])
            assert(isinstance(result, pandas.core.series.Series))
            assert(len(result) == (len(idadf.columns)-1))
            
    def test_su_multiple_target(self, idadf):
        if len(idadf.columns) > 1:
            result = su(idadf, target = [idadf.columns[0],idadf.columns[1]])
            assert(isinstance(result, pandas.core.frame.DataFrame))
            assert(len(result.columns) == 2)
            assert(len(result.index) == len(idadf.columns))
            
    def test_su_one_target_one_feature(self, idadf):
        if len(idadf.columns) > 1:
            result = su(idadf, target = idadf.columns[0], features=[idadf.columns[1]])
            assert(isinstance(result, float))
            result2 = su(idadf, target = idadf.columns[1], features=[idadf.columns[0]])
            assert(round(result,3) == round(result2,3)) # symmetry


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#-----------------------------------------------------------------------------
# Copyright (c) 2015, IBM Corp.
# All rights reserved.
#
# Distributed under the terms of the BSD Simplified License.
#
# The full license is in the LICENSE file, distributed with this software.
#-----------------------------------------------------------------------------

"""
Test module for IdaGeoDataFrame
"""
from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
from future import standard_library
standard_library.install_aliases()

import pandas
import pytest
import six

from copy import deepcopy

from nzpyida import IdaDataFrame
from nzpyida import IdaSeries
from nzpyida import IdaGeoDataFrame
from nzpyida import IdaGeoSeries

@pytest.mark.skipif("'netezza' in config.getvalue('jdbc') or config.getvalue('hostname') != ''")
class Test_IdaGeoDataFrame(object):

    def test_idageodf_set_geometry_error(self, idageodf_county):
        with pytest.raises(KeyError):
            idageodf_county.set_geometry('not a column in the Ida')
        with pytest.raises(TypeError):
            idageodf_county.set_geometry('OBJECTID')

    def test_idageodf_set_geometry_success(self, idageodf_county):
        geometry_colname = 'SHAPE'
        idageodf_county.set_geometry(geometry_colname)
        assert(isinstance(idageodf_county.geometry, IdaGeoSeries))
        assert(idageodf_county.geometry.column == geometry_colname)

    def test_idageodf_nondestructive_geometry_column_deletion(
            self, idageodf_county):
        geometry_colname = 'SHAPE'
        idageodf_county.set_geometry(geometry_colname)
        assert(idageodf_county.geometry.column == geometry_colname)
        del(idageodf_county[geometry_colname])
        with pytest.raises(AttributeError):
            idageodf_county.geometry

    def test_idageodf_geometry_not_set(self, idageodf_county):
        idageodf_county._geometry_colname = None
        with pytest.raises(AttributeError):
            idageodf_county.geometry

    def test_idageodf_fromIdaDataFrame(self, idadf):
        newidageodf = IdaGeoDataFrame.from_IdaDataFrame(idadf)
        assert(isinstance(newidageodf, IdaGeoDataFrame))

    def test_idageodf_column_projection_IdaGeoSeries(self, idageodf_county):
        indexer = 'OBJECTID'
        idageodf_county.indexer = indexer
        column = 'SHAPE'
        ida = idageodf_county[column]
        assert(isinstance(ida, IdaGeoSeries))
        assert(ida.column == column)
        assert(ida.indexer == indexer)

    def test_idageodf_column_projection_IdaSeries(self, idageodf_county):
        indexer = 'OBJECTID'
        idageodf_county.indexer = indexer
        column = 'NAME'
        ida = idageodf_county[column]
        assert(isinstance(ida, IdaSeries))
        assert(ida.column == column)
        assert(ida.indexer == indexer)

    def test_idageodf_column_projection_IdaGeoDataFrame(self, idageodf_county):
        geometry_colname = 'SHAPE'
        idageodf_county.set_geometry(geometry_colname)
        columns = ['SHAPE', 'OBJECTID']
        ida = idageodf_county[columns]
        assert(isinstance(ida, IdaGeoDataFrame))
        assert(all(ida.columns) == all(columns))
        assert(ida.geometry.column == geometry_colname)

    def test_idageodf_column_projection_IdaDataFrame(self, idageodf_county):
        geometry_colname = 'SHAPE'
        idageodf_county.set_geometry(geometry_colname)
        columns = ['NAME', 'OBJECTID']
        ida = idageodf_county[columns]
        assert(isinstance(ida, IdaDataFrame))
        assert(all(ida.columns) == all(columns))
        with pytest.raises(AttributeError):
            ida.geometry

    def test_idageodf_geospatial_method_call_carried_on_IdaGeoSeries(
            self, idageodf_county):
        geometry_colname = 'SHAPE'
        idageodf_county.set_geometry(geometry_colname)
        attribute = 'area'
        assert(not hasattr(IdaGeoDataFrame, attribute))
        assert(hasattr(IdaGeoSeries, attribute))
        assert(idageodf_county.__getattr__(attribute))

    def test_idageodf_getattr_unresolved(self, idageodf_county):
        with pytest.raises(AttributeError):
            idageodf_county.__getattr__('not_an_attribute')

    def test_idageodf_equals(self, idageodf_customer, idageodf_county):
        idageodf = idageodf_customer
        idageodf.set_geometry('SHAPE')
        idageodf_county.set_geometry('SHAPE')
        assert (isinstance(idageodf.equals(idageodf_county), IdaGeoDataFrame))

    def test_idageodf_distance(self, idageodf_customer,idageodf_county):
        idageodf = idageodf_customer
        idageodf.set_geometry('SHAPE')
        idageodf_county.set_geometry('SHAPE')
        assert (isinstance(idageodf.distance(idageodf_county), IdaGeoDataFrame))

    def test_idageodf_crosses(self,idageodf_customer,idageodf_county):
        idageodf = idageodf_customer
        idageodf.set_geometry('SHAPE')
        idageodf_county.set_geometry('SHAPE')
        assert (isinstance(idageodf.crosses(idageodf_county), IdaGeoDataFrame))

    def test_idageodf_intersects(self, idageodf_customer, idageodf_county):
        idageodf = idageodf_customer
        idageodf.set_geometry('SHAPE')
        idageodf_county.set_geometry('SHAPE')
        assert (isinstance(idageodf.intersects(idageodf_county), IdaGeoDataFrame))

    def test_idageodf_overlaps(self, idageodf_customer, idageodf_county):
        idageodf = idageodf_customer
        idageodf.set_geometry('SHAPE')
        idageodf_county.set_geometry('SHAPE')
        assert (isinstance(idageodf.overlaps(idageodf_county), IdaGeoDataFrame))

    def test_idageodf_touches(self, idageodf_customer, idageodf_county):
        idageodf = idageodf_customer
        idageodf.set_geometry('SHAPE')
        idageodf_county.set_geometry('SHAPE')
        assert (isinstance(idageodf.touches(idageodf_county), IdaGeoDataFrame))

    def test_idageodf_disjoint(self, idageodf_customer, idageodf_county):
        idageodf = idageodf_customer
        idageodf.set_geometry('SHAPE')
        idageodf_county.set_geometry('SHAPE')
        assert (isinstance(idageodf.disjoint(idageodf_county), IdaGeoDataFrame))

    def test_idageodf_contains(self, idageodf_customer,idageodf_county):
        idageodf = idageodf_customer
        idageodf.set_geometry('SHAPE')
        idageodf_county.set_geometry('SHAPE')
        assert (isinstance(idageodf.contains(idageodf_county), IdaGeoDataFrame))

    def test_idageodf_within(self, idageodf_customer, idageodf_county):
        idageodf = idageodf_customer
        idageodf.set_geometry('SHAPE')
        idageodf_county.set_geometry('SHAPE')
        assert (isinstance(idageodf.within(idageodf_county), IdaGeoDataFrame))

    def test_idageodf_mbr_intersects(self, idageodf_customer, idageodf_county):
        idageodf = idageodf_customer
        idageodf.set_geometry('SHAPE')
        idageodf_county.set_geometry('SHAPE')
        assert (isinstance(idageodf.mbr_intersects(idageodf_county), IdaGeoDataFrame))

    def test_idageodf_difference(self, idageodf_customer,idageodf_county):
        idageodf = idageodf_customer
        idageodf.set_geometry('SHAPE')
        idageodf_county.set_geometry('SHAPE')
        assert (isinstance(idageodf.difference(idageodf_county), IdaGeoDataFrame))

    def test_idageodf_intersection(self, idageodf_customer,idageodf_county):
        idageodf = idageodf_customer
        idageodf.set_geometry('SHAPE')
        idageodf_county.set_geometry('SHAPE')
        assert (isinstance(idageodf.intersection(idageodf_county), IdaGeoDataFrame))

    def test_idageodf_union(self, idageodf_customer,idageodf_county):
        idageodf = idageodf_customer
        idageodf.set_geometry('SHAPE')
        idageodf_county.set_geometry('SHAPE')
        assert (isinstance(idageodf.union(idageodf_county), IdaGeoDataFrame))

    def test_idageodf_binary_operation_handler_non_geometry_column(
            self, idageodf_customer,idageodf_county):
        idageodf = idageodf_customer  # ST_POLYGON
        with pytest.raises(TypeError):
            idageodf._binary_operation_handler(
                idageodf_county,
                db2gse_function='DB2GSE.ST_AGEOSPATIALFUNCTION',
                valid_types=['ST_POINT'])

    def test_idageodf_max_distance(self, idageodf_county):
         idageodf = idageodf_county
         idageodf.set_geometry('SHAPE')
         ida1 = idageodf[idageodf['NAME'] == 'Austin']
         ida2 = idageodf[idageodf['NAME'] == 'Kent']
         res = ida1.distance(ida2, 'KILOMETER')
         max_dist = res['RESULT'].max()
         assert(int(max_dist) == 2531)

    def test_idageodf_max_distance_lc(self, idageodf_county_view):
         idageodf = idageodf_county_view
         idageodf.set_geometry('shape')
         ida1 = idageodf[idageodf['name'] == 'Austin']
         ida2 = idageodf[idageodf['name'] == 'Kent']
         res = ida1.distance(ida2, 'kilometer')
         max_dist = res['RESULT'].max()
         assert(int(max_dist) == 2531)


    def test_idageodf_max_distance_mbr(self, idageodf_county):
         idageodf = idageodf_county
         idageodf.set_geometry('SHAPE')
         idageodf['MBR'] = idageodf.mbr()
         idageodf.set_geometry('MBR')
         ida1 = idageodf[idageodf['NAME'] == 'Austin']
         ida2 = idageodf[idageodf['NAME'] == 'Kent']
         res = ida1.distance(ida2, 'KILOMETER')
         max_dist_mbr = res['RESULT'].max()
         assert(int(max_dist_mbr) == 2519)

    def test_idageodf_max_distance_mbr_lc(self, idageodf_county_view):
         idageodf = idageodf_county_view
         idageodf.set_geometry('shape')
         idageodf['mbr'] = idageodf.mbr()
         idageodf.set_geometry('mbr')
         ida1 = idageodf[idageodf['name'] == 'Austin']
         ida2 = idageodf[idageodf['name'] == 'Kent']
         res = ida1.distance(ida2, 'kilometer')
         max_dist_mbr = res['RESULT'].max()
         assert(int(max_dist_mbr) == 2519)

    def test_idageodf_max_area_union(self, idageodf_county):
         idageodf = idageodf_county
         idageodf.set_geometry('SHAPE')
         idageodf['MBR'] = idageodf.mbr()
         idageodf.set_geometry('MBR')
         ida1 = idageodf[idageodf['NAME'] == 'Austin']
         ida2 = idageodf[idageodf['NAME'] == 'Kent']
         ida12_union = ida1.union(ida2)
         ida12_union.set_geometry('RESULT')
         ida12_union['MBR_UNION_AREA'] = ida12_union.area('KILOMETER')
         max_area_union = ida12_union['MBR_UNION_AREA'].max()
         assert(int(max_area_union) == 6596)

    def test_idageodf_max_area_union_lc(self, idageodf_county_view):
         idageodf = idageodf_county_view
         idageodf.set_geometry('shape')
         idageodf['mbr'] = idageodf.mbr()
         idageodf.set_geometry('mbr')
         ida1 = idageodf[idageodf['name'] == 'Austin']
         ida2 = idageodf[idageodf['name'] == 'Kent']
         ida12_union = ida1.union(ida2)
         ida12_union.set_geometry('RESULT')
         ida12_union['mbr_union_area'] = ida12_union.area('kilometer')
         max_area_union = ida12_union['mbr_union_area'].max()
         assert(int(max_area_union) == 6596)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#-----------------------------------------------------------------------------
# Copyright (c) 2015, IBM Corp.
# All rights reserved.
#
# Distributed under the terms of the BSD Simplified License.
#
# The full license is in the LICENSE file, distributed with this software.
#-----------------------------------------------------------------------------

"""
setup.py
"""
# Note: Always prefer setuptools over distutils
from setuptools import setup, find_packages
from codecs import open
import os

# Get the long description from the relevant file


with open('README.rst', 'r', encoding='utf-8') as f:
    longdesc = f.read()


classifiers = [
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 4 - Beta',

        # Indicate who your project is intended for
        'Intended Audience :: Developers',
        'Intended Audience :: Education',
        'Intended Audience :: End Users/Desktop',
        'Intended Audience :: Information Technology',
        'Intended Audience :: Science/Research',
        'Topic :: Software Development :: Build Tools',

        # Pick your license as you wish (should match "license" above)
        'License :: OSI Approved :: BSD License',

        'Operating System :: MacOS :: MacOS X',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: POSIX',

        'Natural Language :: English',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: Implementation :: CPython',

        'Topic :: Database',
        'Topic :: Scientific/Engineering',
        'Topic :: Software Development'
      ]

setup(name='nzpyida',
      version='0.2.2.2',
      install_requires=['pandas','numpy','future','six','pypyodbc','lazy', 'nzpy'],

      extras_require={
        'jdbc':['JayDeBeApi==1.*', 'Jpype1==0.6.3'],
        'test':['pytest', 'flaky==3.4.0'],
        'doc':['sphinx', 'ipython', 'numpydoc', 'sphinx_rtd_theme']
      },
      description='Supports Custom ML/Analytics Execution Inside Netezza',
      long_description=longdesc,
      long_description_content_type='text/markdown',
      author='IBM Corp.',
      author_email='vinay.kasireddy@ibm.com,toni.bollinger@de.ibm.com',
      license='BSD',
      classifiers=classifiers,
      keywords='data analytics database development ibm netezza pandas scikitlearn scalability machine-learning knowledge discovery',
      packages=find_packages(exclude=['docs', 'tests*']),
      package_data={
        'nzpyida.sampledata': ['*.txt']}
     )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2015, IBM Corp.
# All rights reserved.
#
# Distributed under the terms of the BSD Simplified License.
#
# The full license is in the LICENSE file, distributed with this software.
# -----------------------------------------------------------------------------

"""
In-Database User defined functions
"""
from __future__ import print_function
from __future__ import unicode_literals
from __future__ import division
from __future__ import absolute_import

import inspect
import textwrap
from builtins import dict
from future import standard_library

from nzpyida.ae import shaper, result_builder

standard_library.install_aliases()


class NZFunTApply(object):

    def __init__(self, df,  parallel,  output_signature=None, output_table=None, fun_ref=None, code_str=None,  fun_name=None, columns=None, merge_output_with_df=False, id='ID'):
        """
        Constructor for tapply
        """
        self.table_name = df.internal_state.current_state
        self.df = df
        self.db = df._idadb
        self.fun = fun_ref
        self.fun_name =fun_name
        self.code_str = code_str
        self.output_table = output_table
        self.output_signature = output_signature
        self.parallel =parallel
        self.merge_output = merge_output_with_df
        self.id = id

        # convert the columns index object to a list
        if columns:
            self.columns = columns
        else:
            self.columns = self.df.columns.tolist()





    def get_result(self):

        if self.code_str and self.fun_name is None:
            raise Exception("fun_name is required")
        # we need a comma separated string of column values
        columns_string = ",".join(self.columns)


        # get the default ae class coupled with client ml code


        if self.code_str:
         code_string = self.build_udtf_shaper_ae_code_fun_as_str(self.code_str, self.fun_name, self.columns, self.output_signature)
        else :
         code_string = self.build_udtf_shaper_ae_code_fun_as_ref(self.fun, self.columns, self.output_signature)


        # send the code as dynamic variable to ae function
        columns_string = columns_string + ",'CODE_TO_EXECUTE=" + "\"" + code_string + "\"" + "'"



        if not self.parallel:
            ae_name = "nzpyida..py_udtf_host"
        else:
            ae_name ="nzpyida..py_udtf_any"

        query = "select ae_output.* from " + \
                " (select * from " + self.table_name + ") as input_t" + \
                ", table with final (" + ae_name + "(" + columns_string + ")) as ae_output"


        result = result_builder.build_result(self.output_table, self.merge_output, self.db, self.df,
                                             self.output_signature, self.table_name, query, self.id)
        return result





    def build_udtf_shaper_ae_code_fun_as_ref(self, fun, columns, output_signature):

        return self.build_udtf_shaper_ae_code_fun_as_str(inspect.getsource(fun).lstrip(), fun.__name__, columns,
                                                         output_signature)




    def build_udtf_shaper_ae_code_fun_as_str(self, code_str, fun_name, columns, output_signature):
        # we need extra single quotes for correct escaping

        fun_code = code_str

        fun_code = fun_code.replace("'", "''")

        base_code = shaper.get_base_shaper_tapply(columns, fun_name, output_signature)



        run_string = textwrap.dedent(""" BaseShaperUdtf.run()""")
        final_code = base_code + "\n" + textwrap.indent(fun_code, '     ')
        final_code = final_code + "\n" + run_string





        return inspect.cleandoc(final_code)







#!/usr/bin/env python
# -*- coding: utf-8 -*-
#-----------------------------------------------------------------------------
# Copyright (c) 2015, IBM Corp.
# All rights reserved.
#
# Distributed under the terms of the BSD Simplified License.
#
# The full license is in the LICENSE file, distributed with this software.
#-----------------------------------------------------------------------------

'''
This module provides Fisher's Iris flower data set. It provides an object 'iris' which is
a pandas dataframe with 150 rows and the following fields:
    iris['petal_length']
    iris['petal_width']
    iris['sepal_length']
    iris['sepal_width']
    iris['species']

Examples
--------
>>> from nzpyida.sampledata.iris import iris
'''

from __future__ import absolute_import
from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from future import standard_library
standard_library.install_aliases()
from os.path import dirname, join
import pandas as pd

iris = pd.read_csv(join(dirname(__file__), 'iris.txt'))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#-----------------------------------------------------------------------------
# Copyright (c) 2015, IBM Corp.
# All rights reserved.
#
# Distributed under the terms of the BSD Simplified License.
#
# The full license is in the LICENSE file, distributed with this software.
#-----------------------------------------------------------------------------

"""
Test module for statistics for IdaDataFrame Objects
"""

from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
from builtins import object
from builtins import str
from future import standard_library
standard_library.install_aliases()

import numpy
import pandas
import pytest

from nzpyida.statistics import _numeric_stats , _get_percentiles, _get_number_of_nas, _count_level, _count_level_groupby
from nzpyida import IdaDataFrame as IDADF

class Test_PrivateStatisticsMethods(object):

    def test_idadf_numeric_stats_default(self, idadf):
        data = idadf._table_def() # We necessarly have to put the test under this condition
        columns = list(data.loc[data['VALTYPE'] == "NUMERIC"].index)
        assert isinstance(_numeric_stats(idadf, "count", columns), numpy.ndarray)
        assert isinstance(_numeric_stats(idadf, "mean", columns), numpy.ndarray)
        assert isinstance(_numeric_stats(idadf, "median", columns), numpy.ndarray)
        assert isinstance(_numeric_stats(idadf, "std", columns), numpy.ndarray)
        assert isinstance(_numeric_stats(idadf, "var", columns), numpy.ndarray)
        assert isinstance(_numeric_stats(idadf, "min", columns), numpy.ndarray)
        assert isinstance(_numeric_stats(idadf, "max", columns), numpy.ndarray)

    def test_idadf_onecolumn_numeric_numeric_stats_one_column(self, idadf_onecolumn_numeric):
        column = idadf_onecolumn_numeric.columns.tolist()
        assert(isinstance(_numeric_stats(idadf_onecolumn_numeric, "count", column), numpy.float64) |
               isinstance(_numeric_stats(idadf_onecolumn_numeric, "count", column), numpy.int64) )
        assert isinstance(_numeric_stats(idadf_onecolumn_numeric, "mean", column), numpy.float64)
        assert isinstance(_numeric_stats(idadf_onecolumn_numeric, "median", column), numpy.ndarray)
        assert isinstance(_numeric_stats(idadf_onecolumn_numeric, "std", column), numpy.float64)
        assert isinstance(_numeric_stats(idadf_onecolumn_numeric, "var", column), numpy.float64)
        assert isinstance(_numeric_stats(idadf_onecolumn_numeric, "min", column), numpy.float64)
        assert isinstance(_numeric_stats(idadf_onecolumn_numeric, "max", column), numpy.float64)

    def test_idadf_numeric_stats_accuracy(self, idadf):
        pass

    def test_idadf_get_percentiles_default(self, idadf):
        data = idadf._table_def() # We necessarly have to put the test under this condition
        columns = list(data.loc[data['VALTYPE'] == "NUMERIC"].index)
        assert isinstance(_get_percentiles(idadf, 0.5,columns), pandas.core.frame.DataFrame)
        assert isinstance(_get_percentiles(idadf, [0.5],columns), pandas.core.frame.DataFrame)
        assert _get_percentiles(idadf, [0.5],columns).shape[0] == 1
        assert isinstance(_get_percentiles(idadf, [0.3,0.4,0.5],columns), pandas.core.frame.DataFrame)
        assert _get_percentiles(idadf, [0.3,0.4,0.5],columns).shape[0] == 3

    def test_idadf_get_percentiles_accuracy(self, idadf):
        pass

    def test_idadf_get_categorical_stats(self, idadf):
        # FUNCTION TO IMPLEMENT
        pass

    def test_idadf_get_number_of_nas(self, idadf):
        assert isinstance(_get_number_of_nas(idadf, idadf.columns), tuple)
        assert len(_get_number_of_nas(idadf, idadf.columns)) == len(idadf.columns)
        assert len(_get_number_of_nas(idadf, idadf.columns[0])) == 1

    def test_idadf_get_number_of_nas_accuracy(self, idadf):
        pass



    def test_idadf_count_level(self, idadf):
        assert isinstance(_count_level(idadf), tuple)
        assert len(_count_level(idadf)) == len(idadf.columns)
        assert len(_count_level(idadf, idadf.columns)) == len(idadf.columns)

    def test_idadf_count_level_accuracy(self, idadf):
        pass

    def test_idadf_count_level_groupby(self, idadf):
        assert isinstance(_count_level_groupby(idadf), tuple)
        assert(_count_level_groupby(idadf)[0] % 1 == 0)
        assert len(_count_level_groupby(idadf)) == 1
        assert len(_count_level_groupby(idadf, idadf.columns)) == 1

    def test_idadf_count_level_group_by_accuracy(self, idadf):
        pass

    def test_idadf_factor_count(self, idadf):
        pass

    def test_idadf_factor_sum(self, idadf):
        pass

    def test_idadf_factor_avg(self, idadf):
        pass

class Test_DescriptiveStatistics(object):

    def test_idadf_pivot_table(self, idadf):
        pass

    def test_idadf_describe_default(self, idadf):
        to_assert = idadf.describe()
        assert isinstance(to_assert, pandas.core.frame.DataFrame)
        assert (len(to_assert) == 8) & (to_assert.shape[1] <= len(idadf.columns))
        assert all([x in to_assert.index for x in ["count","mean","std", "min", "max"]])

    def test_idadf_describe_custom_percentiles(self, idadf):
        assert len(idadf.describe([0.2,0.3,0.4,0.5,0.6])) == 10
        assert len(idadf.describe([0.5])) == 6

    def test_idadf_describe_bad_parameter_value(self, idadf):
        with pytest.raises(ValueError):
            idadf.describe([0.2,0.3,0.4,0.5,-1])

    def test_idadf_describe_bad_parameter_type(self, idadf):
        with pytest.raises(TypeError):
            idadf.describe("string")

    def test_idadf_quantile_default(self, idadf, df):
        assert str(idadf.quantile()) == str(df.quantile())

    def test_idadf_quantile_custom(self, idadf, df):
        assert all(idadf.quantile([0.2,0.4,0.6,0.8]) == df.quantile([0.2,0.4,0.6,0.8]))

    def test_idadf_quantile_value_out_of_range(self, idadf):
        with pytest.raises(ValueError):
            idadf.quantile([5])
        with pytest.raises(ValueError):
            idadf.quantile([-0.4])
        with pytest.raises(TypeError):
            idadf.quantile(["lol"])
        with pytest.raises(TypeError):
            idadf.quantile("lol")
        with pytest.raises(TypeError):
            idadf.quantile(0.5,0.7)

    def test_idadf_cov(self, idadf, df):
        assert str(idadf.cov()) == str(df.cov())

    def test_idadf_corr(self, idadf, df):
        assert str(idadf.corr()) == str(df.corr())

    def test_idadf_mad(self, idadf, df):
        assert str(idadf.mad()) == str(df.mad())

    # TODO : Fix min and max in python 2
    def test_idadf_min(self, idadf, df):
        assert str(idadf.min()) == str(df.min())

    def test_idadf_max(self, idadf, df):
        assert str(idadf.max()) == str(df.max())

    def test_idadf_count(self, idadf, df):
        assert all(idadf.count() == df.count())

    def test_idadf_count_distinct(self, idadf, df):
        pass

    def test_idadf_std(self, idadf, df):
        assert str(idadf.std()) == str(df.std())

    def test_idadf_var(self, idadf, df):
        assert str(idadf.var()) == str(df.var())

    def test_idadf_mean(self, idadf, df):
        assert str(idadf.mean()) == str(df.mean())

    def test_idadf_sum(self, idadf, df):
        assert str(idadf.sum()) == str(df.sum(numeric_only = True))

    def test_idadf_median(self, idadf, df):
        assert str(idadf.median()) == str(df.median())

    @pytest.mark.parametrize("f",
                             [IDADF.describe,
                              IDADF.cov,
                              IDADF.corr,
                              IDADF.quantile,
                              IDADF.mad,
                              IDADF.min,
                              IDADF.max,
                              IDADF.count,
                              IDADF.count_distinct,
                              IDADF.std,
                              IDADF.var,
                              IDADF.mean,
                              IDADF.sum,
                              IDADF.median,
                             ])
    def test_idadf_statistics_one_column(self, idadf_onecolumn_numeric, f):
        f(idadf_onecolumn_numeric)
